unit GameRender;
{<|Author : Adler3D|>}
{<|e-mail : Adler3D@mail.ru|>}
{<|site : Adler3D.narod.ru|>}
interface
uses
  Windows,eXgine,OpenGL,ASDVector;
procedure DrawRect(X,Y,W,H:Real; Mode:GLenum=GL_QUADS);
procedure DrawRectEx(X,Y,W,H:Real; ScaledX:Real=0; ScaledY:Real=0);
procedure DrawQuad(X,Y,W,H:Real; A:Real=0);
procedure DrawQuadZ(X,Y,W,H:Real; A:Real=0);
procedure DrawRib(const A,B:TVector);
procedure DrawNormal(const A,B:TVector);
procedure DrawLine(const A,B:TVector);
procedure DrawLink(A,B:TVector);
procedure DrawPoint(const P:TVector);
const
  clRed:TRGBA=(R:255; G:0; B:0; A:255);
  clLime:TRGBA=(R:0; G:255; B:0; A:255);
  clBlue:TRGBA=(R:0; G:0; B:255; A:255);
  clWhite:TRGBA=(R:255; G:255; B:255; A:255);
  clBlack:TRGBA=(R:0; G:0; B:0; A:255);
const
  // VBO
  GL_ARRAY_BUFFER_ARB=$8892;
  GL_ELEMENT_ARRAY_BUFFER_ARB=$8893;
  GL_STATIC_DRAW_ARB=$88E4;
  GL_NORMAL_ARRAY=$8075;
  GL_COLOR_ARRAY=$8076;
  GL_VERTEX_ARRAY=$8074;
  GL_TEXTURE_COORD_ARRAY=$8078;
  GL_WRITE_ONLY_ARB=$88B9;
procedure glDrawArrays(mode:GLenum; first,count:GLsizei); stdcall;
procedure glDrawElements(mode:GLenum; count:GLsizei; _type:GLenum; const indices:PGLuint); stdcall;
procedure glEnableClientState(_array:GLenum); stdcall;
procedure glDisableClientState(_array:GLenum); stdcall;
procedure glTexCoordPointer(size:Integer; _type:GLenum; stride:Integer; const _pointer:PGLuint); stdcall;
procedure glVertexPointer(size:Integer; _type:GLenum; stride:Integer; const _pointer:PGLuint); stdcall;
procedure glColorPointer(size:Integer; _type:GLenum; stride:Integer; const _pointer:PGLuint); stdcall;
var
  Zoom:Real;
implementation
{(*}
procedure glDrawArrays(mode:GLenum; first,count:GLsizei); stdcall; external opengl32;
procedure glDrawElements(mode:GLenum; count:GLsizei; _type:GLenum; const indices:PGLuint); stdcall; external opengl32;
procedure glEnableClientState(_array:GLenum); stdcall; external opengl32;
procedure glDisableClientState(_array:GLenum); stdcall; external opengl32;
procedure glTexCoordPointer(size:Integer; _type:GLenum; stride:Integer; const _pointer:PGLuint); stdcall; external opengl32;
procedure glVertexPointer(size:Integer; _type:GLenum; stride:Integer; const _pointer:PGLuint); stdcall; external opengl32;
procedure glColorPointer(size:Integer; _type:GLenum; stride:Integer; const _pointer:PGLuint); stdcall; external opengl32;
{*)}

procedure DrawRect(X,Y,W,H:Real; Mode:GLenum);
begin
  glPushMatrix;
  glTranslated(X,Y,0);
  glBegin(Mode);
  glTexCoord2d(0,0); glVertex2f(0,0);
  glTexCoord2d(1,0); glVertex2f(W,0);
  glTexCoord2d(1,1); glVertex2f(W,H);
  glTexCoord2d(0,1); glVertex2f(0,H);
  glEnd;
  glPopMatrix;
end;

procedure DrawRectEx(X,Y,W,H:Real; ScaledX:Real=0; ScaledY:Real=0);
begin
  if ScaledX=0 then ScaledX:=1 else ScaledX:=W/ScaledX;
  if ScaledY=0 then ScaledY:=1 else ScaledY:=H/ScaledY;
  glPushMatrix;
  glTranslated(X,Y,0);
  glBegin(GL_QUADS);
  glTexCoord2d(0,ScaledY); glVertex2f(0,0);
  glTexCoord2d(ScaledX,ScaledY); glVertex2f(W,0);
  glTexCoord2d(ScaledX,0); glVertex2f(W,H);
  glTexCoord2d(0,0); glVertex2f(0,H);
  glEnd;
  glPopMatrix;
end;

procedure DrawQuad(X,Y,W,H:Real; A:Real=0);
begin
  W:=W/2;
  H:=H/2;
  glPushMatrix;
  glTranslated(X,Y,0);
  glRotated(A,0,0,1);
  glBegin(GL_QUADS);
  glTexCoord2d(0,0); glVertex2f(-W,-H);
  glTexCoord2d(1,0); glVertex2f(W,-H);
  glTexCoord2d(1,1); glVertex2f(W,H);
  glTexCoord2d(0,1); glVertex2f(-W,H);
  glEnd;
  glPopMatrix;
end;

procedure DrawQuadZ(X,Y,W,H:Real; A:Real=0);
begin
  DrawQuad(X,Y,W/Zoom,H/Zoom,A);
end;

procedure DrawNormal(const A,B:TVector);
var
  C,D:TVector;
  L:Real;
begin
  if VectorEqual(A,B) then Exit;
  L:=10/Zoom;
  C.X:=(A.X+B.X)/2; C.Y:=(A.Y+B.Y)/2;
  D:=VectorSub(A,B); D:=VectorAdd(C,VectorSetDlina(MakeVector(-D.Y,D.X),L));
  glBegin(GL_LINES);
  glVertex2dv(@C); glVertex2dv(@D);
  glEnd;
end;

procedure DrawLine(const A,B:TVector);
begin
  glBegin(GL_LINES);
  glVertex2dv(@A); glVertex2dv(@B);
  glEnd;
end;

procedure DrawLink(A,B:TVector);
const
  R:Real=2.4;
  Beta=15;
  CPi:Real=(180-Beta)*(Pi/180);
var
  C:TVector;
  Ang:Real;
begin
  if VectorEqual(A,B) then Exit;
  //Canvas.Ellipse(Round(P.X+R),Round(P.Y+R),Round(P.X-R),Round(P.Y-R));
  DrawLine(A,B);
  Ang:=VectorGetAlfa(VectorSub(B,A));
  C:=VectorAdd(B,MakeVectorEx(Ang-CPi,Beta/Zoom));
  A:=VectorAdd(B,MakeVectorEx(Ang+CPi,Beta/Zoom));
  DrawLine(A,B);
  DrawLine(C,B);
end;

procedure DrawRib(const A,B:TVector);
var
  C,D:TVector;
  L:Real;
begin
  if VectorEqual(A,B) then Exit;
  L:=10/Zoom;
  C.X:=(A.X+B.X)/2;
  C.Y:=(A.Y+B.Y)/2;
  D:=VectorSub(A,B);
  D:=VectorAdd(C,VectorSetDlina(MakeVector(-D.Y,D.X),L));
  glBegin(GL_LINES);
  glVertex2dv(@A); glVertex2dv(@B);
  glVertex2dv(@C); glVertex2dv(@D);
  glEnd;
end;

procedure DrawPoint(const P:TVector);
var
  PS:Real;
begin
  PS:=2/Zoom;
  glBegin(GL_QUADS);
  glVertex2d(P.X-PS,P.Y-PS);
  glVertex2d(P.X+PS,P.Y-PS);
  glVertex2d(P.X+PS,P.Y+PS);
  glVertex2d(P.X-PS,P.Y+PS);
  glEnd;
end;

end.

