unit GameEntity;
{<|Author : Adler3D|>}
{<|e-mail : Adler3D@mail.ru|>}
{<|site : Adler3D.narod.ru|>}
interface
uses
  Windows,eXgine,OpenGL,ASDVector,QuickSprites;
type
  TActionSprite=class;
  TEntity=class;
  TTowerEngine=class;
  TTower=class;
  TBulletEngine=class;
  TBullet=class;
  THomingBullet=class;
  TExtraEngine=class;
  TExtraExplosion=class;
  TAwardEngine=class;
  TAward=class;
  TTextAward=class;
  {#TActionSprite#}
  TEntityEvent=procedure(Sender:TEntity) of object;
  TActionSprite=class(TSprite)
  protected
    procedure DoMove; override;
    procedure DoAction; virtual;
  public
    {#<Field#}
    Tick:Integer; //Counter
    UPA:Integer; //Update per Action
    {#Field>#}
    procedure SetUPA(Value:Integer);
  end;
  {#TEntity#}
  TEntity=class(TActionSprite)
  protected
    procedure DoMove; override;
    procedure DoDraw; override;
  public
    {#<Field#}
    R:Real; //Radius
    C:TRGBA; //Color
    M:Real; //Mass
    P:TVector; //Position
    V:TVector; //Velocity
    A:TVector; //Acceleration
    TexID:Integer; //Texture ID
    Ang:Real; //Angle
    {#Field>#}
    procedure SetMass(Mass:Real);
    procedure SetR(R:Real);
    procedure AddImpulse(Imp:TVector);
    procedure AddAcc(Acc:TVector);
    property AX:Real read A.X write A.X;
    property AY:Real read A.Y write A.Y;
    property VX:Real read V.X write V.X;
    property VY:Real read V.Y write V.Y;
    property PX:Real read P.X write P.X;
    property PY:Real read P.Y write P.Y;
  end;
  {#TTowerEngine#}
  PWeapon=^TWeapon;
  TShotProc=function(Weapon:PWeapon; Owner:TEntity):TBullet of object;
  TWeapon=record
    DoShot:TShotProc;
    Target:TEntity;
    LastBullet:TBullet;
    TexID:Integer;
  end;
  TTowerEngine=class(TSpriteEngine)
  public
    procedure Draw; override;
    function AddTower(X,Y:Real; UPA:Integer; Weapon:TWeapon):TTower;
  end;
  {#TTower#}
  TTower=class(TEntity)
  protected
    procedure DoAction; override;
    procedure DoDraw; override;
  public
    {#<Field#}
    Weapon:TWeapon;
    {#Field>#}
  end;
  {#TUserTower#}
  TUserTower=class(TTower)
  protected
    procedure DoMove; override;
  public
    {#<Field#}
    //MapAng:Real; //
    {#Field>#}
  end;
  {#TBulletEngine#}
  TBulletEngine=class(TSpriteEngine)
  public
    procedure Draw; override;
    procedure Collider;
  end;
  {#TBullet#}
  TBullet=class(TEntity)
  protected
    procedure DoMove; override;
  public
    {#<Field#}
    Target:TEntity;
    Owner:TEntity;
    {#Field>#}
    procedure Dead; override;
  end;
  {#THomingBullet#}
  THomingBullet=class(TBullet)
  protected
    procedure DoAction; override;
    procedure DoMove; override;
  public
    {#<Field#}
    IncAng:Real; //Increment angle per update
    {#Field>#}
  end;
  TPusBullet=class(THomingBullet)
  end;
  {#TExtraEngine#}
  TExtraEngine=class(TSpriteEngine)
  public
    function AddExplosion(P:TVector; ForceValue:Real):TExtraExplosion;
  end;
  {#TExtraExplosion#}
  TExtraExplosion=class(TEntity)
  protected
    procedure DoDraw; override;
    procedure DoAction; override;
  public
    {#<Field#}
    ForceValue:Real; //ForceValue
    AC:Integer; //ActionCount
    {#Field>#}
  end;
  {#TAwardEngine#}
  TAwardEngine=class(TSpriteEngine)
  public
    {#<Field#}
    {#Field>#}
    function AddAward(X,Y:Real; OnAction:TEntityEvent; UPA:Integer):TAward;
    function AddText(X,Y:Real; S:string):TTextAward;
    procedure Draw; override;
  end;
  {#TAward#}
  TAward=class(TEntity)
  protected
    procedure DoAction; override;
    procedure DoMove; override;
  public
    {#<Field#}
    OnAction:TEntityEvent;
    AngSpeed:Real; // Angular Speed
    {#Field>#}
  end;
  TTextAward=class(TAward)
  protected
    procedure DoDraw; override;
  public
    {#<Field#}
    Text:string;
    {#Field>#}
  end;

implementation
uses
  GameRender,Tools,TextSystem,Global,ASDLine;

{ TActionSprite }

procedure TActionSprite.DoAction;
begin
  {No Action}
end;

procedure TActionSprite.DoMove;
begin
  inherited;
  if Tick=UPA then
  begin
    DoAction; Tick:=0;
  end else Inc(Tick);
end;

procedure TActionSprite.SetUPA(Value:Integer);
begin
  UPA:=Value; Tick:=Random(Value);
end;

{ TEntity }

procedure TEntity.AddAcc(Acc:TVector);
begin
  AX:=AX+Acc.X;
  AY:=AY+Acc.Y;
end;

procedure TEntity.AddImpulse(Imp:TVector);
begin
  AX:=AX+Imp.X/M;
  AY:=AY+Imp.Y/M;
end;

procedure TEntity.DoDraw;
begin
  tex.Enable(Img[TexID]);
  glColor4ubv(@C);
  DrawQuad(PX,PY,R*2,R*2,Ang);
  tex.Disable;
end;

procedure TEntity.DoMove;
var
  dT:Real;
begin
  inherited;
  dT:=1{/App.UPS};
  PX:=PX+VX*dT+AX*dT*dT/2; PY:=PY+VY*dT+AY*dT*dT/2; //calc Pos
  VX:=VX+AX*dT; VY:=VY+AY*dT; //calc velocity
  AX:=0; AY:=0; //calc Acceleration
end;

procedure TEntity.SetMass(Mass:Real);
const
  Ro=1;
begin
  M:=Mass;
  R:=Sqrt(Mass/(Ro*Pi));
end;

procedure TEntity.SetR(R:Real);
begin
  Self.R:=R;
  M:=Pi*R*R;
end;

{ TTowerEngine }

function TTowerEngine.AddTower(X,Y:Real; UPA:Integer; Weapon:TWeapon):TTower;
begin
  Result:=TTower.Create(Self);
  Result.P:=MakeVector(X,Y);
  Result.UPA:=UPA;
  Result.C:=RGBA(255,255,255,255);
  Result.SetR(24);
  Result.TexID:=6;
  Result.Weapon:=Weapon;
end;

procedure TTowerEngine.Draw;
begin
  //tex.Enable(Img[1]);
  inherited;
  //tex.Disable;
end;

{ TTower }

procedure TTower.DoAction;
begin
  if Assigned(Weapon.DoShot)and(Assigned(Weapon.Target)) then
    if Weapon.Target.Deaded then Weapon.Target:=nil else
      Weapon.LastBullet:=Weapon.DoShot(@Weapon,Self);
end;

procedure TTower.DoDraw;
begin
  inherited;
  if Weapon.Target=nil then Exit;
  tex.Enable(Img[Weapon.TexID]);
  DrawQuad(PX,PY,128,32,VectorGetAlfa(VectorSub(Weapon.Target.P,P))/Pi*180);
  tex.Disable;
end;

{ TUserTower }

procedure TUserTower.DoMove;
var
  _4it:record
    A,V,P:TVector;
  end;
begin
  _4it.A:=A;
  _4it.V:=V;
  _4it.P:=P;
  A:=NulVector;
  V:=NulVector;
  P:=NulVector;
  inherited;
  A:=_4it.A;
  V:=_4it.V;
  P:=_4it.P;
end;

{ TBulletEngine }

procedure TBulletEngine.Collider;
var
  I,J:Integer;
  B:TBullet;
begin
  for J:=0 to Count-1 do
  begin
    B:=TBullet(Items[J]);
    for I:=0 to GSys.LE.Count-1 do
    begin
      if DoCollision(B.P,B.V,GSys.LE.Lines[I],B.R/2) then
      begin
        if (B is TPusBullet)and(B.UPA<>1000) then
        begin
          B.SetUPA(1000);
        end else B.Dead;
      end;
    end;
  end;
end;

procedure TBulletEngine.Draw;
begin
  tex.Enable(Img[1]);
  inherited;
  tex.Disable;
end;

{ THomingBullet }

procedure THomingBullet.DoAction;
const
  StepAng=180*Pi/180/100;
var
  Temp:TVector;
  RA:Real;
begin
  if Target=nil then
  begin
    Target:=TTower(Owner).Weapon.Target; Exit;
  end;
  if Target.Deaded then
  begin
    Target:=nil; Exit;
  end else
  begin
    Temp:=VectorSub(Target.P,P);
    RA:=VectorCompareAlfa(Temp,V);
    if Abs(RA)>StepAng*UPA then
      IncAng:=(RA/Abs(RA))*StepAng else IncAng:=RA/UPA;
  end;
end;

procedure THomingBullet.DoMove;
begin
  inherited;
  V:=VectorAddAlfa(V,IncAng);
  Ang:=VectorGetAlfa(V)*180/pi;
end;

{ TExtraEngine }

function TExtraEngine.AddExplosion(P:TVector; ForceValue:Real):TExtraExplosion;
begin
  Result:=TExtraExplosion.Create(Self);
  Result.P:=P;
  Result.ForceValue:=ForceValue;
  Result.V:=NulVector;
  Result.A:=NulVector;
  Result.SetUPA(5);
  Result.AC:=0;
  //Result.Tick
end;

{ TExtraExplosion }

procedure TExtraExplosion.DoAction;
var
  I:Integer;
  Temp:TEntity;
  Mag:Real;
begin
  if AC>=5 then Dead else Inc(AC);
  for I:=0 to GSys.TE.Count-1 do
  begin
    Temp:=TEntity(GSys.TE[I]);
    if not(Temp is TUserTower) then Continue;
    Mag:=VectorABDlina(Temp.P,P);
    if Mag<8 then Mag:=8 else if Mag>500 then Continue; Mag:=30/Mag;
    Temp.V:=VectorAdd(Temp.V,VectorSetDlina(VectorSub(Temp.P,P),Mag*ForceValue));
  end;
end;

procedure TExtraExplosion.DoDraw;
begin
//  DrawTextQ3(0,PX,PY,'^70_o');
end;

{ TBullet }

procedure TBullet.Dead;
begin
  {GSys.EE.AddExplosion(P,0.5); }GSys.AddExplosion(P,0.5,Self);
  inherited;
end;

procedure TBullet.DoMove;
begin
  inherited;
  Ang:=VectorGetAlfa(V)/pi*180;
end;

{ TAwardEngine }

function TAwardEngine.AddAward(X,Y:Real; OnAction:TEntityEvent; UPA:Integer):TAward;
begin
  Result:=TAward.Create(Self);
  Result.P:=MakeVector(X,Y);
  Result.C:=RGBA(64,128,255,255);
  Result.OnAction:=OnAction;
  Result.SetUPA(UPA);
  Result.SetR(16);
  Result.TexID:=2;
end;

function TAwardEngine.AddText(X,Y:Real; S:string):TTextAward;
begin
  Result:=TTextAward.Create(Self);
  Result.P:=MakeVector(X,Y);
  Result.C:=RGBA(64,255,128,255);
  Result.OnAction:=nil;
  Result.SetUPA(20);
  Result.SetR(16);
  Result.Text:=S;
  Result.AngSpeed:=2;
  Result.TexID:=2;
end;

procedure TAwardEngine.Draw;
begin
  //tex.Enable(Img[2]);
  inherited;
  //tex.Disable(0);
end;

{ TAward }

procedure TAward.DoAction;
begin
  if Assigned(OnAction) then OnAction(Self);
end;

procedure TAward.DoMove;
begin
  inherited; Ang:=Ang+AngSpeed;
end;

{ TTextAward }

procedure TTextAward.DoDraw;
begin
  DrawTextQ3(0,PX,PY-16,'^7'+Text);
  inherited;
end;

end.

