unit DNet;

interface

uses
  windows,
  winsock,

  DUtils,
  DDebug,
  DParser;

const
  ipAny:        string = ''; //    
  ipBroadCast:  string = '255.255.255.255'; //     LAN
  ipLoopBack:   string = '127.0.0.1'; //   

type
TSockApi = class;

TNetAddress = record
  IP: TString;
  Port: Word;
end;

TPacket = TData;

TSocketUDP = class
private
  FSID: winsock.TSocket;
public
  constructor Create(Api: TSockApi; Port: Word = 0);
  function Send(const Address: TNetAddress; const Packet: TPacket): Boolean;
  function Recv(var Address: TNetAddress; var Packet: TPacket): Boolean;
  function Broadcast(const Port: Word; const Packet: TPacket): Boolean;
end;

TSocketTCP = class
private
  FSID: winsock.TSocket;
public
  function Send(const Packet: TPacket): Boolean;
  function Recv(var Packet: TPacket): Boolean;
end;

TTCPOnConnection = procedure (const Address: String; Socket: TSocketTCP) of object;
TServerTCP = class
private
  FOnConnection: TTCPOnConnection;
public
  constructor Create(Api: TSockApi; Port: Word);
  procedure Start;
  procedure Stop;
  property OnConnection: TTCPOnConnection read FOnConnection write FOnConnection;
end;

TSockApi = class
private
  FReady: Boolean;
public
  constructor Create;
  destructor Destroy; override;
end;

function GetWinSockErrorStr(Error: Integer): TString;
function CheckError(R: Integer): Boolean;

function NA(IP: TString; Port: Word = 0): TNetAddress;
function NA2sockaddr(NA: TNetAddress): TSockAddr;
function sockaddr2NA(Addr: TSockAddr): TNetAddress;

implementation

{$IFNDEF FLAG_FPC}{$REGION 'utils'}{$ENDIF}
function GetWinSockErrorStr(Error: Integer): TString;
begin
  case Error of
    WSAEINTR: Result := 'WSAEINTR';
    WSAEBADF: Result := 'WSAEBADF';
    WSAEACCES: Result := 'WSAEACCES';
    WSAEDISCON: Result := 'WSAEDISCON';
    WSAEFAULT: Result := 'WSAEFAULT';
    WSAEINVAL: Result := 'WSAEINVAL';
    WSAEMFILE: Result := 'WSAEMFILE';
    WSAEWOULDBLOCK: Result := 'WSAEWOULDBLOCK';
    WSAEINPROGRESS: Result := 'WSAEINPROGRESS';
    WSAEALREADY: Result := 'WSAEALREADY';
    WSAENOTSOCK: Result := 'WSAENOTSOCK';
    WSAEDESTADDRREQ: Result := 'WSAEDESTADDRREQ';
    WSAEMSGSIZE: Result := 'WSAEMSGSIZE';
    WSAEPROTOTYPE: Result := 'WSAEPROTOTYPE';
    WSAENOPROTOOPT: Result := 'WSAENOPROTOOPT';
    WSAEPROTONOSUPPORT: Result := 'WSAEPROTONOSUPPORT';
    WSAESOCKTNOSUPPORT: Result := 'WSAESOCKTNOSUPPORT';
    WSAEOPNOTSUPP: Result := 'WSAEOPNOTSUPP';
    WSAEPFNOSUPPORT: Result := 'WSAEPFNOSUPPORT';
    WSAEAFNOSUPPORT: Result := 'WSAEAFNOSUPPORT';
    WSAEADDRINUSE: Result := 'WSAEADDRINUSE';
    WSAEADDRNOTAVAIL: Result := 'WSAEADDRNOTAVAIL';
    WSAENETDOWN: Result := 'WSAENETDOWN';
    WSAENETUNREACH: Result := 'WSAENETUNREACH';
    WSAENETRESET: Result := 'WSAENETRESET';
    WSAECONNABORTED: Result := 'WSWSAECONNABORTEDAEINTR';
    WSAECONNRESET: Result := 'WSAECONNRESET';
    WSAENOBUFS: Result := 'WSAENOBUFS';
    WSAEISCONN: Result := 'WSAEISCONN';
    WSAENOTCONN: Result := 'WSAENOTCONN';
    WSAESHUTDOWN: Result := 'WSAESHUTDOWN';
    WSAETOOMANYREFS: Result := 'WSAETOOMANYREFS';
    WSAETIMEDOUT: Result := 'WSAETIMEDOUT';
    WSAECONNREFUSED: Result := 'WSAECONNREFUSED';
    WSAELOOP: Result := 'WSAELOOP';
    {WSAENAMETOOLONG: Result := 'WSAENAMETOOLONG';
    WSAEHOSTDOWN: Result := 'WSAEHOSTDOWN';
    WSASYSNOTREADY: Result := 'WSASYSNOTREADY';
    WSAVERNOTSUPPORTED: Result := 'WSAVERNOTSUPPORTED';
    WSANOTINITIALISED: Result := 'WSANOTINITIALISED';
    WSAHOST_NOT_FOUND: Result := 'WSAHOST_NOT_FOUND';
    WSATRY_AGAIN: Result := 'WSATRY_AGAIN';
    WSANO_RECOVERY: Result := 'WSANO_RECOVERY';
    WSANO_DATA: Result := 'WSANO_DATA';}
  else
    Result := 'UNKNOWN';
  end;  
end;

function CheckError(R: Integer): Boolean;
  var
    Error: Integer;
begin
  Result := R = 0;
  if not Result then begin
    Error := WSAGetLastError;
    SDebug.Log('! DNet: winsock error ' + IntToStr(Error) + ' ' + 
                                          GetWinSockErrorStr(Error));
  end;
end;

function NA(IP: TString; Port: Word = 0): TNetAddress;
  var
    I, P: String;
begin
  if (Port = 0) and (Pos(':', IP) <> 0) then begin
    ParseBinary(IP, ':', I, P);
    IP := I; 
    Port := StrToInt(P);
  end;
  Result.IP := IP;
  Result.Port := Port;
end;

function NA2sockaddr(NA: TNetAddress): TSockAddr;
begin
  if NA.IP = '' then
    Result.sin_addr.S_addr := INADDR_ANY
  else
    Result.sin_addr.S_addr := inet_addr(PChar(NA.IP));
  Result.sin_port        := htons(NA.Port);
  Result.sin_family      := AF_INET;  
  FillChar(Result.sin_zero, SizeOf(Result.sin_zero), 0);
end;

function sockaddr2NA(Addr: TSockAddr): TNetAddress;
begin
  Result.IP   := inet_ntoa(Addr.sin_addr);
  Result.Port := ntohs(Addr.sin_port);
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}

{$IFNDEF FLAG_FPC}{$REGION 'TSocketUDP'}{$ENDIF}
constructor TSocketUDP.Create(Api: TSockApi; Port: Word = 0);
begin
end;

function TSocketUDP.Send(const Address: TNetAddress; const Packet: TPacket): Boolean;
  var
    SockAddr: TSockAddr;
begin
  SockAddr := NA2SockAddr(Address);
  Result := SendTo(FSID, Packet.GetData, Packet.GetSize, 0, 
                   @SockAddr, SizeOf(SockAddr)) <> 0;
end;

function TSocketUDP.Recv(var Address: TNetAddress; var Packet: TPacket): Boolean;
  var
    SockAddr: TSockAddr;
    Count: Integer;
    AddrLen: Integer;
begin
  AddrLen := SizeOf(SockAddr);
  Count := RecvFrom(FSID, Packet.GetData, Packet.GetSize, 0,
                    @SockAddr, @AddrLen);
  Result := Count <= 0;
  if Count < 0 then
    Exit; // error
  if Count = 0 then
    Exit; // no packets
  Address := SockAddr2NA(SockAddr);
end;

function TSocketUDP.Broadcast(const Port: Word; const Packet: TPacket): Boolean;
begin
  Result := Send(NA(ipBroadCast, Port), Packet);
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}

{$IFNDEF FLAG_FPC}{$REGION 'TSocketTCP'}{$ENDIF}
function TSocketTCP.Send(const Packet: TPacket): Boolean;
begin
  Result := winsock.Send(FSID, Packet.GetData, Packet.GetSize, 0) <> 0;
end;

function TSocketTCP.Recv(var Packet: TPacket): Boolean;
  var
    Count: Integer;
begin
  Count := winsock.Recv(FSID, Packet.GetData, Packet.GetSize, 0);
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}

{$IFNDEF FLAG_FPC}{$REGION 'TServerTCP'}{$ENDIF}
constructor TServerTCP.Create(Api: TSockApi; Port: Word);
begin
end;

procedure TServerTCP.Start;
begin
end;

procedure TServerTCP.Stop;
begin
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}

{$IFNDEF FLAG_FPC}{$REGION 'TSockApi'}{$ENDIF}
constructor TSockApi.Create;
  Var
    winsock_version: Word;
    winsock_data: TWSADATA;
begin
  winsock_version := MAKEWORD(1, 0);
  FReady := CheckError(WSAStartup(winsock_version, winsock_data));
end;

destructor TSockApi.Destroy;
begin
  CheckError(WSACleanup);
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}

end.
