unit DGuiConsoleEx;

interface

uses
  DUtils,
  DParser,
  DGuiConsole;

type
TCommandList = class(TCommandValue)
protected
  procedure PrintTypeDescription; override;
  function GetValue: String; override;
  procedure SetValue(Value: String); override;
end;

TArrayOfFloat = array[0..1] of Single;
PArrayOfFloat = ^TArrayOfFloat;

TCommandVecf = class(TCommandValue)
private
  FComponents: Integer;
  FMin: Single;
  FMax: Single;
  FDigits: Integer;
protected
  procedure PrintTypeDescription; override;
  function GetValue: String; override;
  procedure SetValue(Value: String); override;
public
  constructor Create(Name: String; 
                     Adr: Pointer; 
                     Components: Integer = 4;
                     Min: Single = -1000;
                     Max: Single =  1000;
                     Digits: Integer = 6);
end;

implementation

procedure TCommandList.PrintTypeDescription;
begin
  Console.Print(['var ' + ColorName + StandColor + ': |#87.af.87|list', StandColor, ' of strings separated by comma']);
end;

function TCommandList.GetValue: String;
  var
    I: Integer;
begin
  for I := 0 to TListOfString(Adr).Last do
    Result := Result + TListOfString(Adr)[I] + ', ';
  Delete(Result, Length(Result) - 1, 2);
end;

procedure TCommandList.SetValue(Value: String);
  var
    I: Integer;
    Items: TArrayOfString;
begin
  Value := DeleteLeftSpaces(DeleteRightSpaces(Value));
  ParseList(Value, ',', Items);
  TListOfString(Adr).Count := Length(Items);
  for I := 0 to High(Items) do
    TListOfString(Adr)[I] := DeleteLeftSpaces(DeleteRightSpaces(Items[I]));
  inherited;
end;

procedure TCommandVecf.PrintTypeDescription;
begin
  Console.Print(['var ' + ColorName + StandColor + ': |#87.af.87|vec', FComponents, 'f|#8c.8c.8c.ff| in range [', ConstColor, FloatToStr(FMin, FDigits), '..', FloatToStr(FMax, FDigits), StandColor, '] with ', ConstColor, FDigits, StandColor, ' digits']);
end;

function TCommandVecf.GetValue: String;
  var
    I: Integer;
begin
  Result := '(';
  for I := 0 to FComponents - 1 do
    Result := Result + FloatToStr(PArrayOfFloat(Adr)[I], FDigits) + ', ';
  Delete(Result, Length(Result), 1);
  Result[Length(Result)] := ')';
end;

procedure TCommandVecf.SetValue(Value: String);
  var
    I: Integer;
    Temp: array of Single;
    Items: TArrayOfString;
begin
  ParseList(Value, ',', Items);
  SetLength(Temp, FComponents);
  if Length(Items) <> FComponents then begin
    Console.Print(['|#87.00.00.ff|The vector "|#', ColorToHex(Color), '|' 
                   + Name + '|#87.00.00.ff|" must have |#87.af.af|', FComponents,
                   '|#87.00.00.ff| components.']);
    Exit;
  end;
  for I := 0 to High(Items) do begin
    Temp[I] := StrToFloat(Items[I]);
    if (Temp[I] < FMin)or(Temp[I] > FMax) then begin
      Console.Print(['|#87.00.00.ff|Component ', IntToStr(I),
                     'must be in range [|#87.af.af|', 
                     FloatToStr(FMin, FDigits), '|#87.00.00.ff|..|#87.af.af|', 
                     FloatToStr(FMax, FDigits), '|#87.00.00.ff|]']);
      Exit;      
    end;
  end;    
  for I := 0 to High(Temp) do
    PArrayOfFloat(Adr)[I] := Temp[I];
  inherited;
end;

constructor TCommandVecf.Create;
begin
  inherited Create(Name, Adr);
  FComponents := Components;
  FMin := Min;
  FMax := Max;
  FDigits := Digits;
end;

end.
