unit DXMLExecutor;

interface

uses
  DUtils,
  DDebug,
  DXML;

type
TXMLExecutor = class;

TXMLExeProc = procedure(Exe: TXMLExecutor; Item: PXMLItem; Into: Boolean) of object;

TXMLPare = record
  Tag: TString;
  Proc: TXMLExeProc;
end;

TXMLExecutor = class
private
  FTags: array of TXMLPare;
  FPushed: Integer;
public
  constructor Create;
  procedure AddTag(Tag: TString; Proc: TXMLExeProc);
  procedure SetDefault(Proc: TXMLExeProc);
  procedure SetOnText(Proc: TXMLExeProc);
  procedure Execute(Item: PXMLItem); overload;
  procedure Execute(XML: TXML); overload;
  procedure Push;
  procedure Pop;
end;

implementation

constructor TXMLExecutor.Create;
begin
  SetLength(FTags, 0);
  FPushed := -1;
end;

procedure TXMLExecutor.AddTag(Tag: TString; Proc: TXMLExeProc);
begin
  SetLength(FTags, Length(FTags) + 1);
  FTags[High(FTags)].Tag := Tag;
  FTags[High(FTags)].Proc := Proc;
end;

procedure TXMLExecutor.SetDefault(Proc: TXMLExeProc);
begin
  AddTag('<', Proc);
end;

procedure TXMLExecutor.SetOnText(Proc: TXMLExeProc);
begin
  AddTag('>', Proc);
end;

procedure TXMLExecutor.Execute(Item: PXMLItem);
  var
    I, J: Integer;
    Pushed: Integer;
    Executed: Boolean;
begin
  if Item.Typ = xitText then begin
    for I := High(FTags) downto 0 do
      if FTags[I].Tag = '>' then begin
        FTags[I].Proc(Self, Item, True);
        Exit;
      end;
  end;
  Push;
  Pushed := FPushed;
  Executed := False;
  for I := High(FTags) downto 0 do
    if FTags[I].Tag = Item.Text then begin
      FTags[I].Proc(Self, Item, True);
      with Item^ do
        for J := 0 to High(Content) do
          Execute(Content[J]);
      FTags[I].Proc(Self, Item, False);
      Executed := True;
      Break;
    end;
  if not Executed then
    for I := High(FTags) downto 0 do
      if FTags[I].Tag = '<' then begin
        FTags[I].Proc(Self, Item, True);
        with Item^ do
          for J := 0 to High(Content) do
            Execute(Content[J]);
        FTags[I].Proc(Self, Item, False);
        Executed := True;
        Break;
      end;
  Pop;
  SetLength(FTags, Pushed + 1);
end;

procedure TXMLExecutor.Execute(XML: TXML);
begin
  Execute(XML.Root);
end;

procedure TXMLExecutor.Push;
begin
  FPushed := High(FTags);
end;

procedure TXMLExecutor.Pop;
begin
end;

end.
