unit DBossDoj;

interface

uses
  DUtils,
  DMath,
  DGame;

type
TBossItem = class(TLineEnemy)
private
  FDelta: TVec2f;
  FColor: TVec4f;
public
  constructor Create(Delta: TVec2f);
  destructor Destroy; override;
  procedure Update; override;
end;

TBossDoj = class(TEnemy)
private
  FWidth: Integer;
  FHeight: Integer;
  FLast: Integer;
  FAction: Integer;
protected
  procedure Stream; override;
public
  constructor Create(Pos: TVec2f);
  procedure Update; override;
  procedure Activate;
  procedure Deactivate;
  procedure Collide(Player: TPlayer); override;
end;

var
  BossDamaged: Integer;
  BossPos: TVec2f;
  BossCount: Integer;

implementation

//{{{ TBossItem
constructor TBossItem.Create(Delta: TVec2f);//{{{
begin
  inherited Create;
  FDelta := Delta;
  //     ...
  FColor.W := 666;
end;
//}}}
destructor TBossItem.Destroy;//{{{
begin
  BossDamaged := Game.Time;
  Dec(BossCount);
  inherited;
end;
//}}}
procedure TBossItem.Update;//{{{
begin
  if FColor.W = 666 then
    FColor := Color;
  Pos := BossPos + FDelta;
  if Game.Time - BossDamaged < 100 then
    Color := Vec4f(1, 1, 1, 1)
  else
    Color := FColor;
end;
//}}}
//}}}
//{{{ TBossDoj
procedure TBossDoj.Stream;//{{{
begin
  inherited;
  Stream(@FWidth, SizeOf(FWidth));
  Stream(@FHeight, SizeOf(FHeight));
  Stream(@BossCount, SizeOf(BossCount));
  Stream(FLast);
  Stream(FAction);
end;
//}}}
constructor TBossDoj.Create(Pos: TVec2f);//{{{
  var
    Enemy: TEnemy;
    I, J: Integer;
begin
  inherited Create;
  Self.Pos := Pos;
  FWidth := 15;
  FHeight := 15;
  BossCount := FWidth * FHeight;
  for I := 0 to FWidth - 1 do
    for J := 0 to FHeight - 1 do begin
      Enemy := TBossItem.Create(Vec2f((I-FWidth/2)*22, (J-FHeight/2)*22));
      //Enemy.Pos := Rect.A + Vec2f(I*22, J*22);
      Enemy.Vel := FVel;
      Enemy.Color := (I*Vec4f(0.8, 0.2, 0.2, 1.0) + 
                      (FWidth-I-1)*Vec4f(0.2, 0.8, 0.2, 1.0))*(1/FWidth);
      Game.AddEnemy(Enemy);
    end;
  FDamagable := False;
  BossDamaged := Game.Time;
end;
//}}}
procedure TBossDoj.Update;//{{{
  var
    I: Integer;
    Enemy: TEnemy;
begin
  {
  if Len2f(Game.Player.Pos - Pos) > 200 + BossCount then
    Pos := Pos + Norm2f(Game.Player.Pos - Pos) * 5;
  if Pos.X > 5000 - BossCount then
    Pos.X := 5000 - BossCount
  else if Len2f(Pos - Game.Player.Pos) < 200 then
    Pos := Pos - Norm2f(Game.Player.Pos - Pos) * 5;
  BossPos := Pos;
  }
  if Pos.Y > 1000 then begin
    Pos := Vec2f(Game.Player.Pos.X, -800);
  end;
  
  Pos := Pos + Vec2f(0, 15);
  BossPos := Pos;
  if BossCount < 20 then
    Game.GoToScene('win');
  if Abs(Pos.Y-Game.Player.Pos.Y) < 15/2 then begin  
  //if Game.Time - FLast > 2000 then begin
    for I := 0 to 20 do begin
      Enemy := TLineEnemy.Create;
      Enemy.Pos := Rect.A - Vec2f(0, 200);
      Enemy.Vel := Norm2f(Game.Player.Pos - Enemy.Pos)*5 
                          + Vec2f(Random, Random);
      Enemy.Color := Vec4f(0.8, 0.8, 0.2, 1.0);
      Game.AddEnemy(Enemy);
    end;
    FLast := Game.Time;
  end;
end;
//}}}
procedure TBossDoj.Activate;//{{{
begin
end;
//}}}
procedure TBossDoj.Deactivate;//{{{
begin
end;
//}}}
procedure TBossDoj.Collide(Player: TPlayer);//{{{
begin
end;
//}}}
//}}}

initialization
  Factory.Regist(TBossItem);
  Factory.Regist(TBossDoj);
end.
