unit e_textures;

interface

uses
  Windows, OpenGL, e_types, e_utils, pngimage;

procedure SetTexture(texture : gluint);
function  EmptyTexture(size : integer; smooth : boolean) : gluint;

procedure SetSphericalTexGen(spherical : boolean);

procedure TranslateTexture(x, y : single);
procedure UnTranslateTexture;
procedure ScaleTexture(x, y : single);
procedure UnScaleTexture;
procedure RotateTexture(angle : single);
procedure UnRotateTexture;

function  LoadPNGTexture(filename: String) : gLuint;
procedure TexturesInitialize;

type
  PByteArray = ^TByteArray;
  TByteArray = array[0..32767] of Byte;

const
  GL_CLAMP_TO_EDGE = $812F;  

var
  texture_circle, texture_bonus, texture_arrow,
  texture_player, texture_rocket1, texture_rocket2, texture_rocket3,

  texture_smoke, texture_explosion,
  texture_back1, texture_back2, texture_back5,
  texture_wheel_back, texture_wheel_front, texture_wheel_trap,
  texture_object1, texture_object2, texture_object3, texture_object4 : gluint;
  texture_enemy1, texture_enemy2, texture_enemy3 : array[1..3] of gluint;
  
  texture_time             : single = 0;
  texture_last_rotation    : single;
  texture_last_translation : TPoint = (x : 0; y : 0);
  texture_last_scalation   : TPoint = (x : 0; y : 0);
  texture_map_blood : gluint;

implementation

function  gluBuild2DMipmaps(Target: GLenum; Components, Width, Height: GLint; Format, atype: GLenum; Data: Pointer): GLint; stdcall; external glu32;
procedure glGenTextures(n: GLsizei; var textures: GLuint); stdcall; external opengl32;
procedure glBindTexture(target: GLenum; texture: GLuint); stdcall; external opengl32;

procedure SetTexture;
begin
glBindTexture(GL_TEXTURE_2D, texture);
end;

function EmptyTexture(size : integer; smooth : boolean) : gluint;
var
txtnumber : glUint;
pData : Pointer;
begin
GetMem(pData, size * size * 4);

glGenTextures(1, txtnumber);
SetTexture(txtnumber);
glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, size, size, 0,
             GL_RGBA, GL_UNSIGNED_BYTE, pData);

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

result :=txtNumber;
end;   

procedure SetSphericalTexGen;
begin
glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
if spherical then
  begin
    glEnable(GL_TEXTURE_GEN_S);
    glEnable(GL_TEXTURE_GEN_T);
  end
else
  begin
    glDisable(GL_TEXTURE_GEN_S);
    glDisable(GL_TEXTURE_GEN_T);
  end;
end;

procedure TranslateTexture;
begin
glMatrixMode(GL_TEXTURE);
glTranslatef(x, y, 0);
glMatrixMode(GL_MODELVIEW);
texture_last_translation := GetPoint(x, y);
end;

procedure UnTranslateTexture;
begin
with texture_last_translation do TranslateTexture(-x, -y);
end;

procedure ScaleTexture;
begin
glMatrixMode(GL_TEXTURE);
glScalef(x, y, 1);
glMatrixMode(GL_MODELVIEW);
texture_last_scalation := GetPoint(x, y);
end;

procedure UnScaleTexture;
begin
with texture_last_scalation do ScaleTexture(1 / x, 1 / y);
end;

procedure RotateTexture;
begin
glMatrixMode(GL_TEXTURE);
glRotatef(angle, 0, 0, 1);
glMatrixMode(GL_MODELVIEW);
texture_last_rotation := angle;
end;

procedure UnRotateTexture;
begin
glMatrixMode(GL_TEXTURE);
glRotatef(-texture_last_rotation, 0, 0, 1);
glMatrixMode(GL_MODELVIEW);
end;

function CreatePNGTexture(Width, Height: Word; pData: Pointer): Integer;
var
  Texture: GLuint;
begin
  glGenTextures(1, Texture);
  glBindTexture(GL_TEXTURE_2D, Texture);
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);  {Texture blends with object background}
  //glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);  {Texture does NOT blend with object background}

  { Select a filtering type. BiLinear filtering produces very good results with little performance impact
    GL_NEAREST               - Basic texture (grainy looking texture)
    GL_LINEAR                - BiLinear filtering
    GL_LINEAR_MIPMAP_NEAREST - Basic mipmapped texture
    GL_LINEAR_MIPMAP_LINEAR  - BiLinear Mipmapped texture
  } 

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR); { only first two can be used }
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR); { all of the above can be used }

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);  

  gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, Width, Height, GL_RGBA, GL_UNSIGNED_BYTE, pData);

  result := Texture;
end;

function LoadPNGTexture(Filename: String) : GLuint;
const
  LoadFromResource = false;
var
  Data : Array of DWORD;
  W, Width : Integer;
  H, Height : Integer;
  AlphaPtr: PByte;
  PNG : TPngObject;
  Texture : gluint;
begin
  if not FileExists(filename) then exit;

  PNG := TPngObject.Create;
  PNG.LoadFromFile(Filename);

  if PNG.Header.ColorType = COLOR_PALETTE then
    begin
      PNG.free;
      exit;
    end;

  Width := PNG.Width;
  Height := PNG.Height;
  SetLength(Data, Width * Height);

  if (PNG.Header.ColorType = COLOR_RGB) or
     (PNG.Header.ColorType = COLOR_GRAYSCALE) then
  begin
    For H := 0 to Height - 1 do
      For W := 0 to Width - 1 do
        Data[W + (H * Width)] := $FF000000 or PNG.Pixels[W, (Height-1)-H];
  end else
  if (PNG.Header.ColorType = COLOR_RGBALPHA) or
     (PNG.Header.ColorType = COLOR_GRAYSCALEALPHA) then
  begin
    for H := 0 to Height - 1 do
    begin
      AlphaPtr := PByte(PNG.AlphaScanline[(Height-1)-H]);
      for W := 0 to Width - 1 do
      begin
        Data[W + (H * Width)] := (($FF000000 and (AlphaPtr^ shl 24)) or PNG.Pixels[W, (Height-1)-H]);
        Inc(AlphaPtr);
      end;
    end;
  end;

  PNG.free;

  Texture := CreatePNGTexture(Width, Height, @Data[0]);

  Result := Texture;
end;

procedure TexturesInitialize;
begin
texture_circle            := LoadPNGTexture('textures/circle.png');
texture_bonus             := LoadPNGTexture('textures/bonus.png');
texture_arrow             := LoadPNGTexture('textures/arrow.png');

texture_player            := LoadPNGTexture('textures/player.png');
texture_rocket1           := LoadPNGTexture('textures/rocket1.png');
texture_rocket2           := LoadPNGTexture('textures/rocket2.png');
texture_rocket3           := LoadPNGTexture('textures/rocket3.png');

texture_enemy1[1]         := LoadPNGTexture('textures/enemy1_1.png');
texture_enemy1[2]         := LoadPNGTexture('textures/enemy1_2.png');
texture_enemy1[3]         := LoadPNGTexture('textures/enemy1_3.png');
texture_enemy2[1]         := LoadPNGTexture('textures/enemy2_1.png');
texture_enemy2[2]         := LoadPNGTexture('textures/enemy2_2.png');
texture_enemy2[3]         := LoadPNGTexture('textures/enemy2_3.png');
texture_enemy3[1]         := LoadPNGTexture('textures/enemy3_1.png');
texture_enemy3[2]         := LoadPNGTexture('textures/enemy3_2.png');
texture_enemy3[3]         := LoadPNGTexture('textures/enemy3_3.png');

texture_back1             := LoadPNGTexture('textures/back1.png');
texture_back2             := LoadPNGTexture('textures/back2.png');
texture_back5             := LoadPNGTexture('textures/back5.png');

texture_smoke             := LoadPNGTexture('textures/smoke.png');
texture_explosion         := LoadPNGTexture('textures/explosion.png');

texture_object1           := LoadPNGTexture('textures/object1.png');
texture_object2           := LoadPNGTexture('textures/object2.png');
texture_object3           := LoadPNGTexture('textures/object3.png');
texture_object4           := LoadPNGTexture('textures/object4.png');

texture_wheel_back        := LoadPNGTexture('textures/wheel_back.png');
texture_wheel_front       := LoadPNGTexture('textures/wheel_front.png');
texture_wheel_trap        := LoadPNGTexture('textures/wheel_trap.png');
end;

end.
