unit e_towers;

interface

uses OpenGL, e_types, e_textures, e_models, e_utils, e_weapons, e_sound;

type
  TTower = class
    mode   : byte;
    pos    : TPoint;
    trap   : TPoint;
    angle  : single;
    creature : TChuck;
    speed  : single;
    length : single;
    size   : single;
  public
    constructor Create(_mode : byte; _pos : TPoint; _angle : single);
    procedure   RenderBack;
    procedure   RenderFront;
    procedure   Update(count : single);
  end;

implementation

uses e_game;

constructor TTower.Create(_mode : byte; _pos : TPoint; _angle : single);
begin
mode      := _mode;
pos       := _pos;
creature  := nil;
angle     := _angle;
speed     := 0;
length    := 3;
size      := 1;

trap.x := pos.x + length * cos(angle / 180 * pi);
trap.y := pos.y + length * sin(angle / 180 * pi);
end;

procedure TTower.RenderBack;
begin
glColor4f(1, 1, 1, 1);

glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(length * 2, length * 2, 1);
SetTexture(texture_wheel_back);
glCallList(model_quad);

glRotatef(angle, 0, 0, 1);
SetTexture(texture_wheel_front);
glCallList(model_quad);
glPopMatrix;
end;

procedure TTower.RenderFront;
begin
glColor4f(1, 1, 1, 1);

glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(length * 2, length * 2, 1);
glRotatef(angle, 0, 0, 1);
SetTexture(texture_wheel_trap);
glCallList(model_quad);
glPopMatrix;
end;

procedure TTower.Update(count : single);
var
i : integer;
s : single;
begin
angle := angle + speed * count * 0.01;
NormalizeAngle(angle);
// ,    
trap.x := pos.x + (length - size * 0.7) * cos((angle + speed) / 180 * pi);
trap.y := pos.y + (length - size * 0.7) * sin((angle + speed) / 180 * pi);

if creature <> nil then
  if (not creature.trapped) or creature.dead then
    creature := nil;

if creature <> nil then
  begin
    SqrAspirePoint(creature.pos, trap, count * 0.01);
    AspireAngle(creature.angle, angle + 90 * sign(speed), count * 0.005 / pi * 180, s);
    NormalizeAngle(creature.angle);
    exit;
  end
else
  SqrAspire(speed, 0, count * 0.003);

for i := 0 to high(game.bullets) do
  if game.bullets[i] <> nil then
    if CollideCircles(trap, game.bullets[i].pos, size, game.bullets[i].size) then
      begin
        creature := game.bullets[i];
        creature.trapped := true;
        s := creature.angle - angle;
        NormalizeAngle(s);
        speed := 20 * sign(s);
        exit;
      end;
end;

end.

