#include <object.h>
#include <core.h>

#include <hge.h>
#include <hgeresource.h>
#include <hgesprite.h>
#include <hgefont.h>
#include <hgeparticle.h>
#include <Box2D.h>

CMace::CMace(HGE *Hge ,b2World *MyWorld, CGameObjectManager *MyObjectManager, CGameInfo *MyGameInfo) : CGameObject(Hge,MyWorld,MyObjectManager,MyGameInfo) {	
	speed = 1500.0f;
	Color = 0x00ffffff;
	Alpha = 0xff000000;

	BodyInfo = new CBodyInfo;
	BodyInfo->Team = 2;
	BodyInfo->isCollide = false;

	b2BodyDef bodyDef;	
	b2CircleDef ShapeDef;

	bodyDef.linearDamping = 1.5f;
	bodyDef.angularDamping = 1.5f;
	bodyDef.position.Set(40.0f, 40.8f);
	body = World->CreateBody(&bodyDef);
	ShapeDef.radius = 1.6f;
	ShapeDef.density = 1.0f;
	ShapeDef.filter.groupIndex = -1;
	ShapeDef.userData = BodyInfo;
	ShapeDef.localPosition.Set(0.f,0.f);
	shape = body->CreateShape(&ShapeDef);
	body->SetMassFromShapes();
}

CMace::~CMace(void) {
	delete par;
	delete sprPar;
	delete sprBody;
	hge->Texture_Free(texPar);
	hge->Texture_Free(texBody);
	World->DestroyBody(body);
	hge->Effect_Free(sndCollision);
	hge=0;
}

bool CMace::Init(void) {
		sndCollision = hge->Effect_Load("data/sound/collision.wav");
		texPar = hge->Texture_Load("data/graphics/particles.png");
		texBody = hge->Texture_Load("data/graphics/mace.png");
		if(!sndCollision ||!texPar ||!texBody)
		{
			MessageBox(NULL, "Can't load #1", "Error", MB_OK | MB_ICONERROR | MB_APPLMODAL);
			hge->System_Shutdown();
			hge->Release();
			return false;
		}

		sprBody = new hgeSprite(texBody, 0, 0, hge->Texture_GetWidth(texBody), hge->Texture_GetHeight(texBody));
		sprBody->SetHotSpot(hge->Texture_GetWidth(texBody)/2.f,hge->Texture_GetHeight(texBody)/2.f);

		// Create and set up a particle system
		sprPar = new hgeSprite(texPar, 0, 0, 32, 32);
		sprPar->SetBlendMode(BLEND_COLORMUL | BLEND_ALPHAADD | BLEND_NOZWRITE);
		sprPar->SetHotSpot(16,16);

		par = new hgeParticleSystem("data/particle/particle_burn.psi",sprPar);
		b2Vec2 pos = body->GetPosition();
		pos.operator *=(GAME_GLOBAL_SIZE);
		par->FireAt(pos.x,pos.y);
	return true;
}

void CMace::Update(float delta) {
	b2Vec2 pos = body->GetPosition();
	pos.operator *=(GAME_GLOBAL_SIZE);
	//par->info.nEmission=(int)(dx*dx+dy*dy)*2;
	par->MoveTo(pos.x,pos.y);
	par->Update(delta);
	if (BodyInfo->isCollide) 
	{
		if(!hge->Channel_IsPlaying(sndChannelCollision)) sndChannelCollision = hge->Effect_Play(sndCollision);
		BodyInfo->isCollide = false;
	}
}

void CMace::Render(void) {
	par->Render();

	b2Vec2 pos = body->GetPosition();
	float32 ang = body->GetAngle();
	pos.operator *=(GAME_GLOBAL_SIZE);
	sprBody->SetColor(0x00555555+Alpha);
	sprBody->RenderEx(pos.x+3,pos.y+3,ang,0.5f);
	sprBody->SetColor(Color+Alpha);
	sprBody->RenderEx(pos.x,pos.y,ang,0.5f);
}