#include <object.h>
#include <core.h>

#include <hge.h>
#include <hgeresource.h>
#include <hgesprite.h>
#include <hgefont.h>
#include <hgeparticle.h>
#include <Box2D.h>

CPlayer::CPlayer(HGE *Hge ,b2World *MyWorld, CGameObjectManager *MyObjectManager, CGameInfo *MyGameInfo) : CGameObject(Hge,MyWorld,MyObjectManager,MyGameInfo) {	
	Color = 0x00ffffff;
	Alpha = 0xff000000;
	body = NULL;
	speed = 1500.0f;
	DelayDamage = 0.5f;
	TimerDamage = 0.f;
	life = 100.f;
	
	BodyInfo = new CBodyInfo;
	BodyInfo->Team = 0;
	BodyInfo->isCollide = false;

	b2BodyDef bodyDef;	
	bodyDef.linearDamping = 10.f;
	bodyDef.angularDamping = 10.f;
	bodyDef.position.Set(40.0f, 30.0f);
	body = World->CreateBody(&bodyDef);
	b2CircleDef ShapeDef;
	ShapeDef.radius = 1.6f;
	ShapeDef.density = 6.0f;
	ShapeDef.filter.groupIndex = -1;
	ShapeDef.userData = BodyInfo;
	ShapeDef.localPosition.Set(0.f,0.f);
	shape = body->CreateShape(&ShapeDef);
	body->SetMassFromShapes();

	// 
	CGameObject *obj;
	ObjectManager->AddObject(obj = new CHandle(hge,World,ObjectManager,GameInfo));
	b2Vec2 a1(0.f,0.f);
	b2Vec2 a2(0.f,-0.8f);
	Joing(body,obj->bodyHead,a1,a2);

	// 
	CGameObject *obj2;
	ObjectManager->AddObject(obj2 = new CMace(hge,World,ObjectManager,GameInfo));
	a1.Set(0.f,0.4f);
	a2.Set(0.f,-1.6f);
	obj->Joing(obj->bodyEnd,obj2->body,a1,a2);

	hge->Input_SetMousePos(400,300);
}

CPlayer::~CPlayer(void) {
	delete par;
	delete fnt;
	delete sprPar;
	delete sprBody;
	hge->Texture_Free(texPar);
	hge->Texture_Free(texBody);
	hge->Effect_Free(sndCollision);
	World->DestroyBody(body);
	hge=0;
}

bool CPlayer::Init(void) {
		sndCollision = hge->Effect_Load("data/sound/collision.wav");
		texPar = hge->Texture_Load("data/graphics/particles.png");
		texBody = hge->Texture_Load("data/graphics/player.png");
		if(!sndCollision ||!texPar ||!texBody)
		{
			MessageBox(NULL, "Can't load #1", "Error", MB_OK | MB_ICONERROR | MB_APPLMODAL);
			hge->System_Shutdown();
			hge->Release();
			return false;
		}

		sprBody = new hgeSprite(texBody, 0, 0, hge->Texture_GetWidth(texBody), hge->Texture_GetHeight(texBody));
		sprBody->SetHotSpot(hge->Texture_GetWidth(texBody)/2.f,hge->Texture_GetHeight(texBody)/2.f);

		fnt=new hgeFont("data/fnt/font1.fnt");

		sprPar = new hgeSprite(texPar, 0, 0, 32, 32);
		sprPar->SetBlendMode(BLEND_COLORMUL | BLEND_ALPHAADD | BLEND_NOZWRITE);
		sprPar->SetHotSpot(16,16);\

		par = new hgeParticleSystem("data/particle/particle_go_player.psi",sprPar);
		parHit = new hgeParticleSystem("data/particle/hit_player.psi",sprPar);
		b2Vec2 pos = body->GetPosition();
		pos.operator *=(GAME_GLOBAL_SIZE);
		par->FireAt(pos.x,pos.y);
	return true;
}

void CPlayer::Update(float delta) {
	b2Vec2 pos = body->GetPosition();
	pos.operator *=(GAME_GLOBAL_SIZE);
	//par->info.nEmission=(int)(dx*dx+dy*dy)*2;
	par->MoveTo(pos.x,pos.y);
	par->Update(delta);
	if (parHit) parHit->MoveTo(pos.x,pos.y);
	parHit->Update(delta);

	float mx,my;
	hge->Input_GetMousePos(&mx,&my);
	b2Vec2 lp(0.f,-0.8f);
	b2Vec2 p = body->GetWorldPoint(lp);	
	b2Vec2 f((mx-400)*speed,(my-300)*speed);
	body->ApplyForce(f,p);

	hge->Input_SetMousePos(400,300);

	TimerDamage += delta;
	if (TimerDamage > DelayDamage)
	{
		if (BodyInfo->isCollide)
		{
			TimerDamage = 0.f;
			if (life>0.0f) 
			{
				life -= 10.0f;
				BodyInfo->isCollide = false;
				
				b2Vec2 pos = body->GetPosition();
				pos.operator *=(GAME_GLOBAL_SIZE);
				parHit->FireAt(pos.x,pos.y);
			}
			else 
			{
				//isAlive = false;
				GameInfo->isGameOver = true;
			}
				
		}
		BodyInfo->isCollide = false;
	}
}

void CPlayer::Render(void) {
	par->Render();

	b2Vec2 pos = body->GetPosition();
	float32 ang = body->GetAngle();
	pos.operator *=(GAME_GLOBAL_SIZE);
	sprBody->SetColor(0x00555555+Alpha);
	sprBody->RenderEx(pos.x+3,pos.y+3,ang,0.5f);
	sprBody->SetColor(Color+Alpha);
	sprBody->RenderEx(pos.x,pos.y,ang,0.5f);

	fnt->SetColor(0xff000000);
	fnt->printf(20.f,65.f,HGETEXT_LEFT,"life %1.f",life);
	fnt->SetColor(Color+Alpha);
	fnt->printf(20.f-3.f,65.f-3.f,HGETEXT_LEFT,"life %1.f",life);

	parHit->Render();
}