#include "Enemy.h"

#include <windows.h>
#include <gl/gl.h>
#include "Resources.h"
#include "../Sound/squall.h"
#include "../Math/Collisions.h"

#include "World.h"

vec2 getRandomPoint ( float offset )
{
	float o = offset * 0.5f;
	return vec2 ( o + frand ( ( float ) WINDOW_WIDTH - offset ), o + frand ( ( float ) WINDOW_HEIGHT - offset ) );
}

/*
 ************************************************************************
 */
void Rocket :: render ()
{
	if ( active )
	{
		glPushMatrix();

		glTranslatef(p.x, p.y, 0.0f);

		glBindTexture ( GL_TEXTURE_2D, tex_rocket );
		float angle = Angle ( v );
		glRotatef(angle * rad2deg, 0.0f, 0.0f, 1.0f);

		glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 1.0f); glVertex2f( -rocketSize, -rocketSize);
		glTexCoord2f(0.0f, 0.0f); glVertex2f( -rocketSize, rocketSize);
		glTexCoord2f(1.0f, 0.0f); glVertex2f( rocketSize, rocketSize);
		glTexCoord2f(1.0f, 1.0f); glVertex2f( rocketSize, -rocketSize);
		glEnd();

		glPopMatrix();
	}
}

void Rocket :: update ( float dt, Player * player  )
{
	if ( active )
	{
		p += v * dt;

		vec2 f = normalize ( player->p - p );
		v = normalize ( v ) * rocketSpeed + f * 0.3f * dt;

		life -= dt;
	}
	if ( frand () < 0.5f )
		smoke->addParticle ( p, 3.0f, 15.0f );
}

/*
 ************************************************************************
 */

void RocketBig :: render ()
{
	if ( active )
	{
		glPushMatrix();

		glTranslatef(p.x, p.y, 0.0f);

		glBindTexture ( GL_TEXTURE_2D, tex_rocket_big );
		float angle = Angle ( v );
		glRotatef(angle * rad2deg, 0.0f, 0.0f, 1.0f);

		glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 1.0f); glVertex2f( -rocketSize, -rocketSize);
		glTexCoord2f(0.0f, 0.0f); glVertex2f( -rocketSize, rocketSize);
		glTexCoord2f(1.0f, 0.0f); glVertex2f( rocketSize, rocketSize);
		glTexCoord2f(1.0f, 1.0f); glVertex2f( rocketSize, -rocketSize);
		glEnd();

		glPopMatrix();
	}
}

void RocketBig :: update ( float dt, Player * player )
{
	if ( active )
	{
		p += v * dt;

		vec2 f = normalize ( player->p - p );
		v = normalize ( v ) * rocketBigSpeed + f * 0.6f * dt;

		life -= dt;
	}
	if ( frand () < 0.5f )
		smoke->addParticle ( p, 4.0f, 7.0f );
}

/*
 ************************************************************************
 */

void Bullet :: render ()
{
	if ( active )
	{
		glPushMatrix();

		glTranslatef(p.x, p.y, 0.0f);

		glBindTexture ( GL_TEXTURE_2D, tex_bullet );
		float angle = Angle ( v );
		glRotatef(angle * rad2deg, 0.0f, 0.0f, 1.0f);

		glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 1.0f); glVertex2f( -bulletSize, -bulletSize);
		glTexCoord2f(0.0f, 0.0f); glVertex2f( -bulletSize, bulletSize);
		glTexCoord2f(1.0f, 0.0f); glVertex2f( bulletSize, bulletSize);
		glTexCoord2f(1.0f, 1.0f); glVertex2f( bulletSize, -bulletSize);
		glEnd();

		glPopMatrix();
	}	
}

void Bullet :: update ( float dt, Player * player  )
{
	if ( active )
	{
		p += v * dt;

		life -= dt;
		if ( life <= 0.0f )
		{
			active = false;
		}
	}
}

/*
 ************************************************************************
 */

void EnemyLaser :: render ()
{
	glPushMatrix();

	glTranslatef(p.x, p.y, 0.0f);

	glPushMatrix ();
	glRotatef ( tick, 0.0f, 0.0f, 1.0f );
	glBindTexture ( GL_TEXTURE_2D, tex_enemy_1_inner );

	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 1.0f); glVertex2f( -enemySize, -enemySize);
	glTexCoord2f(0.0f, 0.0f); glVertex2f( -enemySize, enemySize);
	glTexCoord2f(1.0f, 0.0f); glVertex2f( enemySize, enemySize);
	glTexCoord2f(1.0f, 1.0f); glVertex2f( enemySize, -enemySize);
	glEnd();
	glPopMatrix ();


	glBindTexture ( GL_TEXTURE_2D, tex_enemy_outer );

	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 1.0f); glVertex2f( -enemySize, -enemySize);
	glTexCoord2f(0.0f, 0.0f); glVertex2f( -enemySize, enemySize);
	glTexCoord2f(1.0f, 0.0f); glVertex2f( enemySize, enemySize);
	glTexCoord2f(1.0f, 1.0f); glVertex2f( enemySize, -enemySize);
	glEnd();
	glPopMatrix();



	glPushMatrix ();
	glTranslatef ( aimPos.x, aimPos.y, 0.0f );
	glRotatef ( -tick, 0.0f, 0.0f, 1.0f );

	glBindTexture ( GL_TEXTURE_2D, tex_enemy_1_inner );

	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 1.0f); glVertex2f( -enemySize, -enemySize);
	glTexCoord2f(0.0f, 0.0f); glVertex2f( -enemySize, enemySize);
	glTexCoord2f(1.0f, 0.0f); glVertex2f( enemySize, enemySize);
	glTexCoord2f(1.0f, 1.0f); glVertex2f( enemySize, -enemySize);
	glEnd();
	glPopMatrix ();


	if ( isShoot )
	{
		glColor4f ( 1.0f , 0.0f, 0.0f, ( enemy1MaxShootTime - shootTime ) / enemy1MaxShootTime );

		glPushMatrix ();
		glTranslatef ( p.x, p.y, 0.0f );
		glRotatef ( Angle ( aimPos - p ) * rad2deg, 0.0f, 0.0f, 1.0f );
		glBindTexture ( GL_TEXTURE_2D, tex_point );

		float len = length ( p - aimPos );

		glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 1.0f); glVertex2f( 0.0f, -1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex2f( 0.0f, 1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex2f( len, 1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex2f( len, -1.0f);
		glEnd();

		glPopMatrix ();

		glColor4f ( 1.0f, 1.0f, 1.0f, 1.0f );
	}

#ifdef DEBUG
	glDisable ( GL_TEXTURE_2D );
	glBegin ( GL_POINTS );
	glVertex2f ( targetPoint.x, targetPoint.y );
	glEnd ();
	glEnable ( GL_TEXTURE_2D );
#endif
}

void EnemyLaser :: update ( float dt )
{
	tick += dt * 2.0f;
	p += v * dt;
	enemySize += sinf ( phase + tick * 0.1f ) * 0.2f * dt;

	if ( reloadTime > 0.0f )
		reloadTime -= dt;

	if ( isShoot == false &&  length ( p - player->p ) > 300.0f )
	{
		vec2 f = normalize ( targetPoint - p );
		v = normalize ( v ) * 0.4f + f * 0.01f;
	}
	else
	{
		v = vec2 ( 0.0f );
	}

	if ( length ( p - targetPoint ) < enemyShockSize * 2.0f && isShoot == false )
		targetPoint = getRandomPoint ( 100.0f );


	if ( isShoot )
	{
		shootTime -= dt;
		if ( shootTime <=0 )
		{
			isShoot = false;
			reloadTime = 100.0f;

			world.resolveBullets ( 1, &laser );	
			SQUALL_Sample_Play ( snd_plasma_shot, false, false, true );
		}
	}


	if ( length ( p - player->p ) < 300.0f )
	{
		if ( length ( aimPos - player->p ) > 3.5f )
		{
			if ( isShoot == false && reloadTime <= 0.0f )
			{
				aimPos += normalize ( player->p - aimPos ) * 6.0f * dt;
			}
		}
		else
		{
			if ( isShoot == false )
			{
				aimPos = player->p;
				isShoot = true;
				shootTime = enemy1MaxShootTime;
				SQUALL_Sample_Play ( snd_plasma_charge, false, false, true );
			}
		}
	}
	else
	{
		if ( isShoot == false )
			if ( length ( p - aimPos ) > 5.5f )
				aimPos += normalize ( p - aimPos ) * 8.0f * dt;
			else
				aimPos = p;
	}
}


/*
 ************************************************************************
 */
void EnemyRocket :: render ()
{
	glPushMatrix();

	glTranslatef(p.x, p.y, 0.0f);

	const float pointSize = 2.0f;
	float offX = cosf ( phase + tick * 0.06f ) * ( enemySize - electronOffset );
	float offY = sinf ( phase + tick * 0.06f ) * ( enemySize - electronOffset );
	glBindTexture ( GL_TEXTURE_2D, tex_point );

	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 1.0f); glVertex2f( -pointSize + offX, -pointSize + offY);
	glTexCoord2f(0.0f, 0.0f); glVertex2f( -pointSize + offX, pointSize + offY);
	glTexCoord2f(1.0f, 0.0f); glVertex2f( pointSize + offX, pointSize + offY);
	glTexCoord2f(1.0f, 1.0f); glVertex2f( pointSize + offX, -pointSize + offY);
	glEnd();

	offX = cosf ( phase + tick * 0.05f ) * ( enemySize + electronOffset );
	offY = sinf ( phase + tick * 0.05f ) * ( enemySize + electronOffset );
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 1.0f); glVertex2f( -pointSize + offY, -pointSize + offX);
	glTexCoord2f(0.0f, 0.0f); glVertex2f( -pointSize + offY, pointSize + offX);
	glTexCoord2f(1.0f, 0.0f); glVertex2f( pointSize + offY, pointSize + offX);
	glTexCoord2f(1.0f, 1.0f); glVertex2f( pointSize + offY, -pointSize + offX);
	glEnd();


	glRotatef ( Angle ( player->p - p ) * rad2deg, 0.0f, 0.0f, 1.0f );


	glBindTexture ( GL_TEXTURE_2D, tex_enemy_2_inner );

	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 1.0f); glVertex2f( -enemySize, -enemySize);
	glTexCoord2f(0.0f, 0.0f); glVertex2f( -enemySize, enemySize);
	glTexCoord2f(1.0f, 0.0f); glVertex2f( enemySize, enemySize);
	glTexCoord2f(1.0f, 1.0f); glVertex2f( enemySize, -enemySize);
	glEnd();


	glBindTexture ( GL_TEXTURE_2D, tex_enemy_outer );

	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 1.0f); glVertex2f( -enemySize, -enemySize);
	glTexCoord2f(0.0f, 0.0f); glVertex2f( -enemySize, enemySize);
	glTexCoord2f(1.0f, 0.0f); glVertex2f( enemySize, enemySize);
	glTexCoord2f(1.0f, 1.0f); glVertex2f( enemySize, -enemySize);
	glEnd();

	glPopMatrix();

#ifdef DEBUG
	glDisable ( GL_TEXTURE_2D );
	glBegin ( GL_POINTS );
	glVertex2f ( targetPoint.x, targetPoint.y );
	glEnd ();
	glEnable ( GL_TEXTURE_2D );
#endif
}

void EnemyRocket :: update ( float dt )
{
	tick += dt * 2.0f;
	p += v * dt;

	if ( reloadTime > 0.0f )
		reloadTime -= dt;

	if ( length ( p - player->p ) > 300.0f )
	{
		vec2 f = normalize ( targetPoint - p );
		v = normalize ( v ) * 0.2f + f * 0.03f;
	}
	else
	{
		v = vec2 ( 0.0f );
	}

	if ( length ( p - targetPoint ) < enemyShockSize * 2.0f )
		targetPoint = getRandomPoint ( 100.0f );

	if ( length ( p - player->p ) < 300.0f && reloadTime <= 0.0f )
	{
		world.addRocket ( this, p, normalize ( player->p - p ) );
		reloadTime = 150.0f;
	}
}

/*
 ************************************************************************
 */
void Enemy3 :: render ()
{
	glPushMatrix();
	{
		glTranslatef(p.x, p.y, 0.0f);

		glPushMatrix ();
		{

			glBindTexture ( GL_TEXTURE_2D, tex_enemy_3_inner );
			glRotatef ( Angle ( player->p - p ) * rad2deg, 0.0f, 0.0f, 1.0f );

			glBegin(GL_QUADS);
			glTexCoord2f(0.0f, 1.0f); glVertex2f( -enemySize, -enemySize);
			glTexCoord2f(0.0f, 0.0f); glVertex2f( -enemySize, enemySize);
			glTexCoord2f(1.0f, 0.0f); glVertex2f( enemySize, enemySize);
			glTexCoord2f(1.0f, 1.0f); glVertex2f( enemySize, -enemySize);
			glEnd();

			glPushMatrix ();
			{

				glBindTexture ( GL_TEXTURE_2D, tex_enemy_3_tentacles );
				glRotatef ( 35.0f - cosf ( phase + tick * 0.1f ) * 20.0f, 0.0f, 0.0f, 1.0f );
				
				glBegin(GL_QUADS);
				glTexCoord2f(0.0f, 1.0f); glVertex2f( -enemySize, -enemySize);
				glTexCoord2f(0.0f, 0.0f); glVertex2f( -enemySize, enemySize);
				glTexCoord2f(1.0f, 0.0f); glVertex2f( enemySize, enemySize);
				glTexCoord2f(1.0f, 1.0f); glVertex2f( enemySize, -enemySize);
				glEnd();
			}
			glPopMatrix ();

			glPushMatrix ();
			{

				glBindTexture ( GL_TEXTURE_2D, tex_enemy_3_tentacles );
				glRotatef ( -35.0f + cosf ( phase + tick * 0.1f ) * 20.0f * animType, 0.0f, 0.0f, 1.0f );
				
				glBegin(GL_QUADS);
				glTexCoord2f(0.0f, 1.0f); glVertex2f( -enemySize, -enemySize);
				glTexCoord2f(0.0f, 0.0f); glVertex2f( -enemySize, enemySize);
				glTexCoord2f(1.0f, 0.0f); glVertex2f( enemySize, enemySize);
				glTexCoord2f(1.0f, 1.0f); glVertex2f( enemySize, -enemySize);
				glEnd();
			}
			glPopMatrix ();
		}
		glPopMatrix ();
	}
	glPopMatrix();
}

void Enemy3 :: update ( float dt )
{
	tick += dt;
	p += v * dt;

	if ( length ( p - player->p ) > 300.0f )
	{
		v = normalize ( player->p - p ) * enemy1Speed;
	}
	else
		v *= 0.99f;

	if ( reloadTime > 0.0f )
		reloadTime -= dt;

	if ( length ( p - player->p ) < 300.0f && reloadTime <= 0.0f )
	{
		world.addBullet ( this, p, normalize ( player->p - p ) * bulletSpeed );
		reloadTime = 100.0f;
	}
}

/*
 ************************************************************************
 */
void EnemyShock :: render ()
{
	glPushMatrix();

	glTranslatef(p.x, p.y, 0.0f);

	glRotatef ( Angle ( v ) * rad2deg, 0.0f, 0.0f, 1.0f );

	glBindTexture ( GL_TEXTURE_2D, tex_enemy_4 );

	glBegin (GL_QUADS);
	glTexCoord2f(0.0f, 1.0f); glVertex2f( -enemySize, -enemySize);
	glTexCoord2f(0.0f, 0.0f); glVertex2f( -enemySize, enemySize);
	glTexCoord2f(1.0f, 0.0f); glVertex2f( enemySize, enemySize);
	glTexCoord2f(1.0f, 1.0f); glVertex2f( enemySize, -enemySize);
	glEnd ();


	glColor3f ( 0.48f, 0.86f, 1.0f );
	anim.render ( vec2 ( 0.0f, 0.0f ), 0.0f );
	glColor3f ( 1.0f, 1.0f, 1.0f );

	glPopMatrix();

#ifdef DEBUG
	glDisable ( GL_TEXTURE_2D );
	glBegin ( GL_POINTS );
	glVertex2f ( targetPoint.x, targetPoint.y );
	glEnd ();
	glEnable ( GL_TEXTURE_2D );
#endif
}

void EnemyShock :: update ( float dt )
{
	tick += dt;
	p += v * dt;
	anim.update ( dt );

	if ( reloadTime > 0.0f )
		reloadTime -= dt;

	if ( attack == false )
	{
		vec2 f = normalize ( targetPoint - p );
		v = normalize ( v ) + f * 0.2f * dt;
	}
	else
	{
		vec2 f = normalize ( targetPoint - p );
		v = f * 6.0f;
	}


	if ( attack == true )
	{
		bool flag = Circle ( p, enemySize - 8.0f ).colCircle ( player->p, playerSize - 8.0f );
		if ( flag == true  )
		{
			attack = false;
			player->v = normalize ( player->p - p ) * 8.0f;
			player->health -= 20.0f;
			reloadTime = 150.0f;

			SQUALL_Sample_Play ( snd_shock, false, false, true );
			cam.Shake ( 3, 15 );

			PFShock * effect = new PFShock ( p + normalize ( player->p - p ) * enemySize );
			particlesEngine.AddEmitter ( effect );
		}

		if ( flag == false && length ( p - targetPoint ) < enemySize - 8.0f )
		{
			attack = false;
			reloadTime = 150.0f;
		}
	}

	if ( length ( p - targetPoint ) < enemyShockSize * 2.0f && attack == false )
		targetPoint = getRandomPoint ( 100.0f );


	if ( length ( p - player->p ) < 200.0f && attack == false && reloadTime <= 0.0f )
	{
		SQUALL_Sample_Play ( snd_shock_shot, false, false, true );
		attack = true;
		targetPoint = player->p + normalize( player->p - p ) * 50.0f ;
	}
}

/*
 ************************************************************************
 */
void EnemyBigRocket :: render ()
{
	glPushMatrix();

	glTranslatef(p.x, p.y, 0.0f);


	glRotatef ( Angle ( player->p - p ) * rad2deg, 0.0f, 0.0f, 1.0f );
	glBindTexture ( GL_TEXTURE_2D, tex_enemy_big );

	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 1.0f); glVertex2f( -enemySize, -enemySize);
	glTexCoord2f(0.0f, 0.0f); glVertex2f( -enemySize, enemySize);
	glTexCoord2f(1.0f, 0.0f); glVertex2f( enemySize, enemySize);
	glTexCoord2f(1.0f, 1.0f); glVertex2f( enemySize, -enemySize);
	glEnd();


	glBindTexture ( GL_TEXTURE_2D, tex_enemy_outer );

	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 1.0f); glVertex2f( -enemySize, -enemySize);
	glTexCoord2f(0.0f, 0.0f); glVertex2f( -enemySize, enemySize);
	glTexCoord2f(1.0f, 0.0f); glVertex2f( enemySize, enemySize);
	glTexCoord2f(1.0f, 1.0f); glVertex2f( enemySize, -enemySize);
	glEnd();

	glPopMatrix();

#ifdef DEBUG
	glDisable ( GL_TEXTURE_2D );
	glBegin ( GL_POINTS );
	glVertex2f ( targetPoint.x, targetPoint.y );
	glEnd ();
	glEnable ( GL_TEXTURE_2D );
#endif
}

void EnemyBigRocket :: update ( float dt )
{
	tick += dt * 2.0f;
	p += v * dt;

	if ( reloadTime > 0.0f )
		reloadTime -= dt;

	if ( length ( p - player->p ) > 300.0f )
	{
		vec2 f = normalize ( targetPoint - p );
		v = normalize ( v ) * 0.2f + f * 0.06f * dt;
	}
	else
	{
		v = vec2 ( 0.0f );
	}

	if ( length ( p - targetPoint ) < enemyShockSize * 2.0f )
		targetPoint = getRandomPoint ( 100.0f );

	if ( length ( p - player->p ) < 300.0f && reloadTime <= 0.0f )
	{
		world.addBigRocket ( this, p, normalize ( player->p - p ) );
		reloadTime = 400.0f;
	}
}

