#ifndef __ENEMY_H__
#define __ENEMY_H__

#include "../Math/Vector.h"
#include "Player.h"
#include "../Utils/SpriteAnimation.h"
#include "particlesEffects.h"

class Enemy;

vec2 getRandomPoint ( float offset );

/*
 ************************************************************************
 */
class Shell
{
public:
	vec2 p;
	vec2 v;

	float life;
	bool active;

	Enemy * owner;

public:
	virtual void render () = 0;
	virtual void update ( float dt, Player * player ) = 0;
	virtual void init () = 0;
};

/*
 ************************************************************************
 */
class Rocket: public Shell
{
public:
	PFRocketSmoke * smoke;

public:
	Rocket ()
	{
		init ();
	}

	void init ()
	{
		active = true;
		v = vec2 ( 1.0f, 0.0f );
	}

	void render ();
	void update ( float dt, Player * player );
};

/*
 ************************************************************************
 */
class RocketBig: public Shell
{
public:
	PFRocketSmoke * smoke;

public:
	RocketBig ()
	{
		init ();
	}

	void init ()
	{
		active = true;
		v = vec2 ( 1.0f, 0.0f );
	}

	void render ();
	void update ( float dt, Player * player );
};


/*
 ************************************************************************
 */
class Bullet: public Shell
{
public:
	Bullet () { init (); }

	void init ()
	{
		active = true;
	}

	void render ();
	void update ( float dt, Player * player  );
};

/*
 ************************************************************************
 */
class Laser: public Shell
{
public:
	void render () {}
	void update ( float dt, Player * player  ) {}
	void init () {}
};

/*
 ************************************************************************
 */
class Enemy
{
public:
	vec2 p;
	vec2 v;
	float enemySize;
	float tick;
	float phase;

	float reloadTime;
	int enemyType;

	Player * player;				// aim - player

	bool active;

public:
	virtual void render () = 0;
	virtual void update ( float dt ) = 0;
	virtual void init () = 0;
};


/*
 ************************************************************************
 */
							// laser shooter
class EnemyLaser: public Enemy
{
public:
	vec2 aimPos;
	float shootTime;
	bool isShoot;
	Laser laser;
	vec2 targetPoint;

public:
	EnemyLaser ()
	{}

	void init ()
	{
		enemySize = enemysSize;
		shootTime = 0.0f;
		isShoot = false;
		tick = 0.0f; 
		phase = frand ( 10.0f );
		enemyType = 1;
		laser.owner = this;
		v = vec2 ( 0.0f );
		active = true;
		reloadTime = 0.0f;
		targetPoint = getRandomPoint ( 100.0f );
	}

	void render ();
	void update ( float dt );
};


/*
 ************************************************************************
 */
							// rocket shooter
class EnemyRocket: public Enemy
{
public:
	float electronOffset;
	vec2 targetPoint;

public:
	EnemyRocket ()
	{}

	void init ()
	{
		enemySize = enemysSize;
		tick = 0.0f;
		phase = frand ( 10.0f );
		electronOffset = frand ( 4.0f );
		enemyType = 2;
		reloadTime = 0.0f;
		v = vec2 ( 0.0f );
		active = true;
		targetPoint = getRandomPoint ( 100.0f );
	}

	void render ();
	void update ( float dt );
};


/*
 ************************************************************************
 */

class Enemy3: public Enemy
{
public:
	float animType;

public:
	Enemy3 ()
	{}

	void init ()
	{
		enemySize = enemysSize;
		tick = 0.0f;
		phase = frand ( 10.0f );
		enemyType = 3;
		reloadTime = 0.0f;

		if ( frand () <= 0.5f )
			animType = 1.0f;
		else
			animType = -1.0f;

		v = vec2 ( 0.0f );
		active = true;
	}

	void render ();
	void update ( float dt );
};


/*
 ************************************************************************
 */

class EnemyShock: public Enemy
{
public:
	Animation anim;
	vec2 targetPoint;
	bool attack;

public:
	EnemyShock ()
	{
		anim.init ( vec2 ( enemyShockSize * 2.0f ), tex_enemy_4_anim, 128, 128, 4, 2, 64, 64, true, true, true);
	}

	void init ()
	{
		enemySize = enemyShockSize;
		tick = 0.0f;
		phase = frand ( 10.0f );
		enemyType = 4;
		reloadTime = 0.0f;

		v = vec2 ( 0.0f );
		active = true;
		attack = false;
		targetPoint = getRandomPoint ( 100.0f );
	}

	void render ();
	void update ( float dt );
};

/*
 ************************************************************************
 */
							// rocket shooter
class EnemyBigRocket: public Enemy
{
public:
	vec2 targetPoint;

public:
	EnemyBigRocket ()
	{
		init ();
	}

	void init ()
	{
		enemySize = enemysSize;
		tick = 0.0f;
		phase = frand ( 10.0f );
		enemyType = 5;
		v = vec2 ( 0.0f );
		active = true;
		targetPoint = getRandomPoint ( 100.0f );
	}

	void render ();
	void update ( float dt );
};

#endif