#pragma once

#include "Resources.h"
#include "../Utils/GUI.h"

#pragma warning (disable:4996)

enum MenuState
{
	MS_MENU,
	MS_PLAY
};
MenuState menuState = MS_MENU;
int maxLevels;
int currentLevel = 1;


//  GUI
GUIFrame *wndMain,
		 *wndOptions,
		 *wndChooseLevel;

// main window
GUIButton *btnContinue;

// choose level
GUILabel *lblLevel;

// options window
/*GUIButton *btnOk,
		  *btnCancel,
		  *btnNextPermission;

GUILabel *lblPermission,
		 *lblVSync,
		 *lblFullscreen,
		 *lblMusic,
		 *lblSound;
*/

void startNewGame ( bool restartMusic );
//  
void eventNewGame () { gui.setCurrentWindow ( wndChooseLevel );	menuState = MS_MENU; }
void eventContinue () {	menuState = MS_PLAY; SQUALL_Pause ( false ); SQUALL_Sample_Pause ( snd_menu, true ); }
void eventOptions () { gui.setCurrentWindow ( wndOptions );	menuState = MS_MENU; }
void eventExit () {	OE_Quit (); }
void eventStart ()
{
	startNewGame ( true );
}
void eventLeftLevel ()
{
	currentLevel--;
	if ( currentLevel < 1 ) currentLevel = maxLevels;
	String str;
	str.appendInt ( currentLevel );
	lblLevel->setText ( str.getStr () );
}

void eventRightLevel ()
{
	currentLevel++;
	if ( currentLevel > maxLevels ) currentLevel = 1; 
	String str;
	str.appendInt ( currentLevel );
	lblLevel->setText ( str.getStr () );
}


//  GUI
int btnHeight = 30;
int btnTextHeight = 22;
int lblHeight = 25;


GUIButton * createButton ( GUIFrame * wnd, const char * text, vec2 & pos, vec2 & size, POnClickEvent event )
{
	GUIButton * btn = new GUIButton();
	btn->initialize ( pos, size, ( float ) btnTextHeight );
	btn->setTextures ( tex_button_normal, tex_button_roll, tex_button_roll );
	btn->OnClickEvent = event;
	btn->setText ( text );
	wnd->addElement ( btn );
	return btn;
}

GUILabel * createLabel ( GUIFrame * wnd, const char * text, vec2 & pos )
{
	GUILabel * lbl = new GUILabel();
	lbl->initialize ( pos, ( float ) lblHeight);
	lbl->setText ( text ) ;
	wnd->addElement ( lbl );
	return lbl;
}

GUICheckBox * createCheckBox ( GUIFrame * wnd, const char * text, vec2 & pos, vec2 & size )
{
	GUICheckBox * cb = new GUICheckBox();
	cb->initialize ( pos, size, ( float ) lblHeight );
	cb->setText ( text );
	wnd->addElement ( cb );
	return cb;
}

void clearGui ()
{
	gui.clear ();
}

bool isExtInFile(const char *filename, const char *ext)
{
	if( stricmp(strrchr(filename,'.'), ext) == 0)
		return true;
	else
		return false;
}

void getLevelsCount ()
{
	HANDLE hFile;
	WIN32_FIND_DATA winFileData;

	String path;
	char str [1024];
	GetCurrentDirectory ( 1024, str ); 
	path += str;
	path += "\\Levels\\*.*";

	hFile = FindFirstFile ( path.getStr (), &winFileData );
	maxLevels = 0;
	
	do 
	{
		if ( strcmp ( winFileData.cFileName, "." ) != 0 && strcmp ( winFileData.cFileName, ".." ) != 0 && isExtInFile ( winFileData.cFileName, ".lvl" ) == true )
			maxLevels++;
	}
	while ( FindNextFile ( hFile, &winFileData ) );
}

void initGui()
{
	wndMain = new GUIFrame();
	gui.addWindow(wndMain);

	wndOptions = new GUIFrame();
	gui.addWindow(wndOptions);

	wndChooseLevel = new GUIFrame();
	gui.addWindow(wndChooseLevel);

	// main window
	float btnWidth = 180.0f;
	float btnHeight = 30.0f;
	float btnX = WINDOW_WIDTH * 0.5f - btnWidth * 0.5f;
	float btnY = WINDOW_HEIGHT * 0.5f - btnHeight * 2.0f;

	btnContinue = createButton ( wndMain, "Continue", vec2 ( btnX, btnY ), vec2 ( btnWidth, btnHeight ), eventContinue );
	btnContinue->setActive ( false );
	btnY += btnHeight + 10.0f;
	createButton ( wndMain, "New Game", vec2 ( btnX, btnY ), vec2 ( btnWidth, btnHeight ), eventNewGame );
	btnY += btnHeight + 10.0f;
	createButton ( wndMain, "Options", vec2 ( btnX, btnY ), vec2 ( btnWidth, btnHeight ), eventOptions );
	btnY += btnHeight + 10.0f;
	createButton ( wndMain, "Exit", vec2 ( btnX, btnY ), vec2 ( btnWidth, btnHeight ), eventExit );
	btnY += btnHeight + 10.0f;

	// choose level menu
	btnX = WINDOW_WIDTH * 0.5f - 50.0f;
	btnY = WINDOW_HEIGHT * 0.5f - lblHeight * 2.0f;
	createButton ( wndChooseLevel, "Start", vec2 ( WINDOW_WIDTH - btnWidth - 10.0f, WINDOW_HEIGHT - btnHeight - 10.0f ), vec2 ( btnWidth, btnHeight ), eventStart );
	createLabel ( wndChooseLevel, "Level:", vec2 ( btnX, btnY ) );
	lblLevel = createLabel ( wndChooseLevel, "1", vec2 ( 100.0f + btnX, btnY ) );
	createButton ( wndChooseLevel, "<",  vec2 ( 125.0f + btnX, btnY - 13.0f ), vec2 ( 25.0f, 25.0f ), eventLeftLevel );
	createButton ( wndChooseLevel, ">",  vec2 ( 125.0f + btnX, btnY + 13.0f ), vec2 ( 25.0f, 25.0f ), eventRightLevel );
}

