#pragma comment (lib, "opengl32.lib")
#pragma comment (lib, "glu32.lib")
#pragma warning (disable:4996)

#include <windows.h>
#include <gl/gl.h>
#include "Resources.h"

#include "../Omega/OMEGA_code.cpp"
#include "../Omega/OMEGA_functions.h"
#include "../Sound/squall.h"

#include "../libTexture/TextureManager.h"

#include "World.h"
#include "InitGui.h"
#include "drawStaff.h"


Camera cam;
World world;
TextureManager textureManager;
Font font;
GUIEngine gui;


void startNewGame ( bool restartMusic )
{
	String level ( "Levels\\" );
	level.appendInt ( currentLevel );
	level += ".lvl";
	world.loadLevel ( level.getStr () );
	btnContinue->setActive ( true );
	menuState = MS_PLAY;

	if ( restartMusic == true )
		SQUALL_Sample_Play ( snd_background, true, false, true );
	else
		SQUALL_Sample_Pause ( snd_background, false );
	SQUALL_Sample_Pause ( snd_menu, true );
}

void init ()
{
	srand ( GetTickCount() );

	glClearColor ( 0.0, 0.0, 0.0, 1.0 );

	glEnable ( GL_TEXTURE_2D );

	glEnable ( GL_BLEND );
	glBlendFunc ( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

	getLevelsCount ();

	loadTextures ();
	loadSounds ();
	initGui ();
	font.Init ( tex_font );

										// init camera
	cam.width = WINDOW_WIDTH;
	cam.height = WINDOW_HEIGHT;
	cam.maxSizeX = WINDOW_WIDTH;
	cam.maxSizeY = WINDOW_HEIGHT;

	SQUALL_Sample_Play ( snd_menu, true, false, true );	// play music in menu
}

void display ()
{
	glColor4f(1.0f, 1.0f, 1.0f, 0.3f);
	font.drawText(vec2(10, 5), 25, "by ViZgl");
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	switch ( menuState )
	{
	case MS_MENU:
		world.render ();
		drawQuad ( vec2 ( 0.0f, 0.0f ), vec2 ( ( float ) WINDOW_WIDTH, ( float ) WINDOW_HEIGHT ), vec4 ( 0.0f, 0.0f, 0.0f, 0.75f ) );

		gui.render ();

		// 
		glBindTexture ( GL_TEXTURE_2D, tex_cursor );
		drawTexturedQuad ( vec2 ( ( float ) Mouse_X (), ( float ) Mouse_Y () ) - 16.0f, vec2 ( 32.0f ), vec4 ( 1.0f ) );

		break;

	case MS_PLAY:

		world.render ();

		if ( world.player.health <= 0 )
		{
			SQUALL_Sample_Stop ( snd_background );
			font.drawText ( vec2 ( ( float ) WINDOW_WIDTH * 0.5f - 220.0f, ( float ) WINDOW_HEIGHT * 0.5f - 40.0f ), 80.0f, "Game Over", 0.6f );
		}

		break;
	}
}

void animate ()
{
	const float dt = 0.5f;

	switch ( menuState )
	{
	case MS_MENU:
		{
			gui.update ( dt );
			if ( Key_Press( K_ESCAPE ) )
			{
				gui.setCurrentWindow ( wndMain );
				menuState = MS_MENU;
			}

			break;
		}

	case MS_PLAY:
		{
			world.update ( dt );

			cam.centerCam ( world.player.p, dt );

			if ( world.player.health > 0.0f )
			{
				const float acc = 0.5f;
				if (Key_Down(K_UP) || Key_Down(K_W)) world.player.v.y -= dt * acc;
				if (Key_Down(K_DOWN) || Key_Down(K_S)) world.player.v.y += dt * acc;
				if (Key_Down(K_LEFT) || Key_Down(K_A)) world.player.v.x -= dt * acc;
				if (Key_Down(K_RIGHT) || Key_Down(K_D)) world.player.v.x += dt * acc;
				if (Key_Press(K_SHIFT))
				{
					world.useSlowMotion = !world.useSlowMotion;
					world.slowTransitionTime = 1.0f;
					if ( world.useSlowMotion == true )
						SQUALL_Sample_Play ( snd_slow_enable, false, false, true );
					else
						SQUALL_Sample_Play ( snd_slow_disable, false, false, true );
				}
			}

													// exit to menu
			if ( Key_Press( K_ESCAPE ) )
			{
				gui.setCurrentWindow ( wndMain );
				menuState = MS_MENU;
				SQUALL_Pause ( true );
				SQUALL_Sample_Pause ( snd_menu, false );
			}

			break;
		}
	}
}

void quit()
{
	unloadTextures ();
	SQUALL_Free ();
}

int __stdcall WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	LoadOMEGA( "OMEGA.dll" );

	Timer_Create( (void*)animate, 12, 1);

	OE_RegProcedure( SYS_INIT, (void*)init );
	OE_RegProcedure( SYS_DRAW, (void*)display );
	OE_RegProcedure( SYS_QUIT, (void*)quit );

	OE_SetScreenOptions( WINDOW_WIDTH, 
						 WINDOW_HEIGHT, 
						 32, 
						 0, 
						 false, 
						 false );

	OE_SetWndName("Arena by ViZgl");

	OE_Init();

	return 0;
}
