#include "World.h"
#include "../Utils/Tokenizer.h"
#include "../Math/Collisions.h"
#include "../Sound/squall.h"

#include "drawStaff.h"

#pragma warning (disable:4996)


ParticlesEngine particlesEngine;

float slowMotionDt = 0.05f;


void World :: updateWaves ()
{
	float outOffset = 20.0f;
	while ( waves.getCount () > 0 && waves [0].time <= ellapsedTime )
	{
		switch ( waves [0].type )
		{
		case 1: for ( int i = 0; i < waves [0].count; i++ ) addEnemyLaser ( getPointOutside ( outOffset ) );
			break;
		case 2: for ( int i = 0; i < waves [0].count; i++ ) addEnemyRocket ( getPointOutside ( outOffset ) );
			break;
		case 3: for ( int i = 0; i < waves [0].count; i++ ) addEnemy3 ( getPointOutside ( outOffset ) );
			break;
		case 4: for ( int i = 0; i < waves [0].count; i++ ) addEnemyShock ( getPointOutside ( outOffset ) );
			break;
		case 5: for ( int i = 0; i < waves [0].count; i++ ) addEnemyBigRocket ( getPointOutside ( outOffset ) );
		}
		waves.orderedRemove ( 0 );
	}
}

float World :: updateSlowMo ( float dt )
{
	float newDt;

	if ( slowTime < 0.0f && useSlowMotion == true )
	{
		useSlowMotion = false;
		slowTransitionTime = 1.0f;
		SQUALL_Sample_Play ( snd_slow_disable, false, false, true );
	}

	if ( useSlowMotion )
	{
		slowTime -= dt * 0.02f;
		newDt = lerp ( slowMotionDt, dt, slowTransitionTime );
	}
	else
	{
		newDt = lerp ( dt, slowMotionDt, slowTransitionTime );
	}

	if ( slowTransitionTime > 0.0f )
		slowTransitionTime -= dt * 0.1f;

	if ( slowTime < 10.0f && useSlowMotion == false )
	{
		slowTime += dt * slowMotionDt * 0.2f;
	}

	return newDt;
}

void World :: update ( float dt )
{
	if ( player.health > 0.0f )
	{
		dt = updateSlowMo ( dt );

		ellapsedTime += dt * 0.024f;		// 0.012 = x * 0.5 => x = 0.024
		score += dt * 0.024f;
	}
												// update player
	player.update ( dt );
	if ( player.p.x > (float) WINDOW_WIDTH ) player.p.x = (float) WINDOW_WIDTH;
	if ( player.p.x < 0.0f ) player.p.x = 0.0f;
	if ( player.p.y > (float) WINDOW_HEIGHT ) player.p.y = (float) WINDOW_HEIGHT;
	if ( player.p.y < 0.0f ) player.p.y = 0.0f;


	updateWaves ();

												// update game objects
	for ( unsigned int i = 0; i < enemys.getCount (); i++ )
	{
		if ( enemys [i]->active )
			enemys [i]->update ( dt );
	}

	for ( unsigned int i = 0; i < rockets.getCount (); i++ )
	{
		if ( rockets [i]->active )
		{
			if ( rockets [i]->life <= 0.0f )
			{
				destroyRocket ( i );
				continue;
			}
			rockets [i]->update ( dt, &player );
			resolveBullets ( 2, ( Shell* ) rockets [i] );
		}
	}

	for ( unsigned int i = 0; i < bigRockets.getCount (); i++ )
	{
		if ( bigRockets [i]->active )
		{
			if ( bigRockets [i]->life <= 0.0f )
			{
				destroyBigRocket ( i );
				continue;
			}
			bigRockets [i]->update ( dt, &player );
			resolveBullets ( 5, ( Shell* ) bigRockets [i] );
		}
	}

	for ( unsigned int i = 0; i < bullets.getCount (); i++ )
	{
		if ( bullets [i]->active )
		{
			bullets [i]->update ( dt, &player );
			resolveBullets ( 3, ( Shell* ) bullets [i] );
		}
	}

	particlesEngine.Update(dt);
}


void World :: render ()
{
	glEnable(GL_TEXTURE_2D);


	font.drawText ( vec2 ( ( float ) WINDOW_WIDTH - 140.0f, ( float ) WINDOW_HEIGHT - 30.0f ), 25.0f, getFormatTime (), 0.6f );
	font.drawText ( vec2 ( ( float ) WINDOW_WIDTH - 180.0f, 5.0f ), 20.0f, getFormatScore (), 0.6f );
	drawQuad ( vec2 ( 10.0f, ( float ) WINDOW_HEIGHT - 25.0f ), vec2 ( slowTime * 20.0f, 8.0f ), vec4 ( 1.0f, 1.0f, 1.0f, 0.7f ) );
	drawBoardText ();

	glPushMatrix();
	glTranslatef(-cam.p.x, -cam.p.y, 0.0f);

	particlesEngine.RenderBack();

	player.render ();

	for ( unsigned int i = 0; i < enemys.getCount (); i++ )
	{
		if ( enemys [i]->active )	
			enemys [i]->render ();
	}

	for ( unsigned int i = 0; i < rockets.getCount (); i++ )
		rockets [i]->render ();

	for ( unsigned int i = 0; i < bigRockets.getCount (); i++ )
		bigRockets [i]->render ();

	for ( unsigned int i = 0; i < bullets.getCount (); i++ )
		bullets [i]->render ();

	particlesEngine.RenderFront();

	glPopMatrix();
}

void World :: drawBoardText ()
{
	if (  ellapsedTime <= 4.0f && ellapsedTime >= 0.1f )
	{
		glColor4f ( 0.5f, 0.5f, 0.5f, ( 4.0f - ellapsedTime ) * 0.5f );
		font.drawText ( vec2 ( ( float ) WINDOW_WIDTH * 0.5f - 140.0f, ( float ) WINDOW_HEIGHT * 0.5f - 40.0f ), 80.0f, "Ready?", 0.6f );
		glColor4f ( 1.0f, 1.0f, 1.0f, 1.0f );
	}
	if ( ellapsedTime <= 7.0f && ellapsedTime >= 3.0f )
	{
		glColor4f ( 0.6f, 0.6f, 0.6f, ( 7.0f - ellapsedTime ) * 0.8f );
		font.drawText ( vec2 ( ( float ) WINDOW_WIDTH * 0.5f - 50.0f, ( float ) WINDOW_HEIGHT * 0.5f - 40.0f ), 80.0f, "Go", 0.6f );
		glColor4f ( 1.0f, 1.0f, 1.0f, 1.0f );
	}
}

void World :: resolveBullets ( int bulletType, Shell * shell )
{
	for ( unsigned int i = 0; i < enemys.getCount (); i++ )
	{
		if ( enemys [i] != ( Enemy * ) shell->owner && enemys [i]->active == true )
			switch ( bulletType )
			{
			case 1:
				{
					EnemyLaser * shooter = ( EnemyLaser* ) shell->owner;
					bool flag = colLineCircle ( shooter->p, shooter->aimPos, enemys [i]->p, enemysSize );

					if ( flag )
						destroyEnemy ( i );

					break;
				}
			case 2:
				{
					bool flag = Circle ( shell->p, rocketSize * 0.1f ).colCircle ( enemys [i]->p, enemysSize * 0.5f );

					if ( flag )
					{
						destroyEnemy ( i );
						destroyRocket ( rockets.getIndex ( ( Rocket* ) shell ) );

						return;
					}

					break;
				}
			case 3:
				{
					bool flag = Circle ( shell->p, bulletSize ).colCircle ( enemys [i]->p, enemysSize * 0.5f );

					if ( flag )
					{
						destroyEnemy ( i );
						destroyBullet ( bullets.getIndex ( ( Bullet* ) shell ) );

						return;
					}

					break;
				}

			case 5:
				{
					bool flag = Circle ( shell->p, rocketSize * 0.1f ).colCircle ( enemys [i]->p, enemysSize * 0.5f );

					if ( flag )
					{
						destroyEnemy ( i );
						destroyBigRocket ( bigRockets.getIndex ( ( RocketBig* ) shell ) );

						return;
					}

					break;
				}
			}
	}

	switch ( bulletType )
	{
	case 1:
		{
			EnemyLaser * shooter = ( EnemyLaser* ) shell->owner;
			bool flag = colLineCircle ( shooter->p, shooter->aimPos, player.p, playerSize );
			if ( flag )
			{
				player.health -= 20.0f;
				player.v += normalize ( player.p - shell->p ) * 0.6f;
			}

			break;
		}
	case 2:
		{
			bool flag = Circle ( shell->p, rocketSize * 0.1f ).colCircle ( player.p, playerSize * 0.5f );
			if ( flag )
			{
				player.health -= 25.0f;
				destroyRocket ( rockets.getIndex ( ( Rocket* ) shell ) );

				player.v += normalize ( player.p - shell->p );

				return;
			}

			break;
		}
	case 3:
		{
			bool flag = Circle ( shell->p, bulletSize * 0.1f ).colCircle ( player.p, playerSize * 0.5f );
			if ( flag )
			{
				player.health -= 10.0f;
				destroyBullet ( bullets.getIndex ( ( Bullet* ) shell ) );

				player.v += normalize ( player.p - shell->p ) * 0.5f;

				return;
			}

			break;
		}
	case 5:
		{
			bool flag = Circle ( shell->p, rocketSize * 0.1f ).colCircle ( player.p, playerSize * 0.5f );
			if ( flag )
			{
				player.health -= 35.0f;
				destroyBigRocket ( bigRockets.getIndex ( ( RocketBig* ) shell ) );

				player.v += normalize ( player.p - shell->p );

				return;
			}

			break;
		}
	}
}

int World :: findFreeEnemy ( int type )
{
	for ( unsigned int i = 0; i < enemys.getCount (); i++ )
		if ( enemys [i]->active == false && enemys [i]->enemyType == type )
			return i;

	switch ( type )
	{
	case 1: enemys.add ( new EnemyLaser () );
		break;
	case 2: enemys.add ( new EnemyRocket () );
		break;
	case 3: enemys.add ( new Enemy3 () );
		break;
	case 4: enemys.add ( new EnemyShock () );
		break;
	case 5: enemys.add ( new EnemyBigRocket () );
	}

	return enemys.getCount () - 1;
}

int World :: findFreeRocket ()
{
	for ( unsigned int i = 0; i < rockets.getCount (); i++ )
		if ( rockets [i]->active == false )
			return i;

	rockets.add ( new Rocket () );

	return rockets.getCount () - 1;
}

int World :: findFreeBigRocket ()
{
	for ( unsigned int i = 0; i < bigRockets.getCount (); i++ )
		if ( bigRockets [i]->active == false )
			return i;

	bigRockets.add ( new RocketBig () );

	return bigRockets.getCount () - 1;
}

int World :: findFreeBullet ()
{
	for ( unsigned int i = 0; i < bullets.getCount (); i++ )
		if ( bullets [i]->active == false )
			return i;

	bullets.add ( new Bullet () );

	return bullets.getCount () - 1;
}

int World :: addEnemyLaser ( vec2 & pos )
{
	int i = findFreeEnemy ( 1 );
	EnemyLaser * enemy = ( EnemyLaser* ) enemys [i];
	enemy->init ();
	enemy->p = pos;
	enemy->aimPos = pos;
	enemy->player = &player;

	return i;
}

int World :: addEnemyRocket ( vec2 & pos )
{
	int i = findFreeEnemy ( 2 );
	EnemyRocket * enemy = ( EnemyRocket* ) enemys [i];
	enemy->init ();
	enemy->p = pos;
	enemy->player = &player;

	return i;
}

int World :: addEnemy3 ( vec2 & pos )
{
	int i = findFreeEnemy ( 3 );
	Enemy3 * enemy = ( Enemy3* ) enemys [i];
	enemy->init ();
	enemy->p = pos;
	enemy->player = &player;

	return i;
}

int World :: addEnemyShock ( vec2 & pos )
{
	int i = findFreeEnemy ( 4 );
	EnemyShock * enemy = ( EnemyShock* ) enemys [i];
	enemy->init ();
	enemy->p = pos;
	enemy->player = &player;

	return i;
}

int World :: addEnemyBigRocket ( vec2 & pos )
{
	int i = findFreeEnemy ( 5 );
	EnemyBigRocket * enemy = ( EnemyBigRocket* ) enemys [i];
	enemy->init ();
	enemy->p = pos;
	enemy->player = &player;

	return i;
}

void World :: destroyEnemy ( int index )
{
	switch ( enemys [index]->enemyType )
	{
	case 1: score += 4;
		break;
	case 2: score += 7;
		break;
	case 3: score += 2;
		break;
	case 4: score += 15;
		break;
	case 5: score += 20;
	}

	vec2 pos = enemys [index]->p;
	int enemyType = enemys [index]->enemyType;
	enemys [index]->active = false;
	
	PFEnemyExplosion * newEffect = new PFEnemyExplosion ( pos );
	particlesEngine.AddEmitter ( newEffect );

	cam.Shake ( 2, 10 );
	SQUALL_Sample_Play ( snd_enemy_die, false, false, true);
}

void World :: destroyRocket ( int index )
{
	vec2 pos = rockets [index]->p;
	rockets [index]->active = false;

	PFRocketExplosion * newEffect = new PFRocketExplosion ( pos );
	particlesEngine.AddEmitter ( newEffect );

	PFExplosion * secondEffect = new PFExplosion ( pos, 25.0f );
	particlesEngine.AddEmitter ( secondEffect );

	cam.Shake ( 1, 10 );
	SQUALL_Sample_Play ( snd_rocket_explosion, false, false, true);
}

void World :: destroyBigRocket ( int index )
{
	vec2 pos = bigRockets [index]->p;
	bigRockets [index]->active = false;

	PFRocketExplosion * newEffect = new PFRocketExplosion ( pos );
	particlesEngine.AddEmitter ( newEffect );

	PFExplosion * secondEffect = new PFExplosion ( pos, 30.0f );
	particlesEngine.AddEmitter ( secondEffect );

	cam.Shake ( 2, 10 );
	SQUALL_Sample_Play ( snd_rocket_explosion, false, false, true);
}

void World :: destroyBullet ( int index )
{
	vec2 pos = bullets [index]->p;
	bullets [index]->active = false;

	PFBulletExplosion * newEffect = new PFBulletExplosion ( pos );
	particlesEngine.AddEmitter ( newEffect );

	PFExplosion * secondEffect = new PFExplosion ( pos, 20.0f );
	particlesEngine.AddEmitter ( secondEffect );

	cam.Shake ( 0.5, 10 );
	SQUALL_Sample_Play ( snd_bullet_hit, false, false, true);
}

int World :: addRocket ( Enemy * owner, vec2 & pos, vec2 & vel )
{
	int i = findFreeRocket ();
	Rocket * rocket =  rockets [i];
	rocket->init ();
	rocket->life = 150.0f;
	rocket->active = true;
	rocket->p = pos;
	rocket->v = vel;
	rocket->owner = owner;
	rocket->smoke = new PFRocketSmoke ();
	particlesEngine.AddEmitter ( rocket->smoke );

	// play sound
	SQUALL_Sample_Play ( snd_rocket_shot, false, false, true);

	return i;
}

int World :: addBigRocket ( Enemy * owner, vec2 & pos, vec2 & vel )
{
	int i = findFreeBigRocket ();
	RocketBig * rocket =  bigRockets [i];
	rocket->init ();
	rocket->life = 150.0f;
	rocket->active = true;
	rocket->p = pos;
	rocket->v = vel;
	rocket->owner = owner;
	rocket->smoke = new PFRocketSmoke ();
	particlesEngine.AddEmitter ( rocket->smoke );

	// play sound
	SQUALL_Sample_Play ( snd_rocket_shot, false, false, true);

	return i;
}

int World :: addBullet ( Enemy * owner, vec2 & pos, vec2 & vel )
{
	int i = findFreeBullet ();
	Bullet * bullet =  bullets [i];
	bullet->init ();
	bullet->life = 300.0f;
	bullet->active = true;
	bullet->p = pos;
	bullet->v = vel;
	bullet->owner = owner;

	// play sound
	SQUALL_Sample_Play ( snd_bullet_shot, false, false, true);

	return i;
}


int wavesCompare ( const WaveParam & w1, const WaveParam & w2 ) { return w1.time - w2.time; }
void World :: loadLevel ( const char * fileName )
{
	Tokenizer tok;

	clear ();
	tok.setFile ( fileName );

	char * func;
	char * param;

	while ( ( func = tok.next () ) != NULL )
	{
		if ( stricmp ( func, "EnemyLaser" ) == 0 )
		{
			param = tok.next ();
			if ( stricmp ( param, "{" ) == 0 )
			{
				while ( stricmp ( param = tok.next (), "}" ) != 0 )
				{
					WaveParam w;
					w.time = atoi ( param );
					w.type = 1;
					w.count = atoi ( tok.next () );
					waves.add ( w );
				}
			}
		}
		else
		if ( stricmp ( func, "EnemyRocket" ) == 0 )
		{
			param = tok.next ();
			if ( stricmp ( param, "{" ) == 0 )
			{
				while ( stricmp ( param = tok.next (), "}" ) != 0 )
				{
					WaveParam w;
					w.time = atoi ( param );
					w.type = 2;
					w.count = atoi ( tok.next () );
					waves.add ( w );
				}
			}
		}
		else
		if ( stricmp ( func, "Enemy3" ) == 0 )
		{
			param = tok.next ();
			if ( stricmp ( param, "{" ) == 0 )
			{
				while ( stricmp ( param = tok.next (), "}" ) != 0 )
				{
					WaveParam w;
					w.time = atoi ( param );
					w.type = 3;
					w.count = atoi ( tok.next () );
					waves.add ( w );
				}
			}
		}
		else
		if ( stricmp ( func, "EnemyShock" ) == 0 )
		{
			param = tok.next ();
			if ( stricmp ( param, "{" ) == 0 )
			{
				while ( stricmp ( param = tok.next (), "}" ) != 0 )
				{
					WaveParam w;
					w.time = atoi ( param );
					w.type = 4;
					w.count = atoi ( tok.next () );
					waves.add ( w );
				}
			}
		}
		else
		if ( stricmp ( func, "EnemyBigRocket" ) == 0 )
		{
			param = tok.next ();
			if ( stricmp ( param, "{" ) == 0 )
			{
				while ( stricmp ( param = tok.next (), "}" ) != 0 )
				{
					WaveParam w;
					w.time = atoi ( param );
					w.type = 5;
					w.count = atoi ( tok.next () );
					waves.add ( w );
				}
			}
		}
		else
		if ( stricmp ( func, "Slowmotion" ) )
		{
			slowTime = ( float ) atoi ( tok.next () );
		}
	}

	waves.sort ( wavesCompare );
}

