#include "Gui.h"
#include "../Omega/OMEGA_functions.h"




GUIElement*		GUIFrame::focused;		//    
unsigned char* tmpString;



bool keyDown()
{
	for ( int i=0; i<256; i++)
		if ( Key_Press(i) ) 
			return true;

	return false;
}

void GuiRenderQuad(vec2& v1, vec2& v2)
{
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 1.0f); glVertex2f(v1.x, v1.y);
	glTexCoord2f(1.0f, 1.0f); glVertex2f(v1.x + v2.x, v1.y);
	glTexCoord2f(1.0f, 0.0f); glVertex2f(v1.x + v2.x, v1.y + v2.y);
	glTexCoord2f(0.0f, 0.0f); glVertex2f(v1.x, v1.y + v2.y);
	glEnd();
}

void bindTexture(int id)
{
	glBindTexture(GL_TEXTURE_2D, id);
}




// --------------------------------------------------------------------
// GUIElement
//

GUIElement::GUIElement()
{
	fadeScale = 0.1f;
	pos = vec2(0.0f);
	size = vec2(0.0f);
	active = true;
	mode = 0;
	old_mode = 0;
	phase = 0.0f;
	text = "";
	textPos = vec2(0.0f);
}

bool GUIElement::onCursor(vec2 p)
{
	return    (p.x > pos.x) && (p.x < pos.x + size.x) &&
			  (p.y > pos.y) && (p.y < pos.y + size.y) && active;	
}

void GUIElement::setMode(int m)
{
	if (mode != m)
	{
		phase = 1.0f;
		old_mode = mode;
	}

	mode = m;
}

void GUIElement::update(float dt)
{
	float t;

	t = dt * fadeScale;
	phase -= t;

	if (phase < 0.0f) phase = 0.0f;
}

void GUIElement::onClick(){}
void GUIElement::onMouseOn(){}
void GUIElement::onMouseOver(){}
void GUIElement::render(){}
void GUIElement::renderText(){}





// --------------------------------------------------------------------
// GUIPanel
//

void GUIPanel::initialize(vec2& Pos, vec2& Size, float Alpha)
{
	pos = Pos;
	size= Size;
	alpha = Alpha;
}

void GUIPanel::render()
{
	glColor4f(1, 1, 1, alpha);
	bindTexture(texture);
	GuiRenderQuad(pos, size);
}





// --------------------------------------------------------------------
// GUILabel
//

void GUILabel::renderText()
{
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	font.drawText(pos, text_size, text);
}

void GUILabel::initialize(vec2& Pos, float TextSize)
{
	pos = Pos;
	text_size= TextSize;
}





// --------------------------------------------------------------------
// GUIButton
//

void GUIButton::initialize(vec2& Pos, vec2& Size, float textSize)
{
	pos = Pos;
	size= Size;
	text_size = textSize;
	mode     = GUI_MODE_NORMAL;
	old_mode = mode;
}

void GUIButton::onClick()
{
	if (active)
		if ( OnClickEvent != NULL )
			OnClickEvent();
}

void GUIButton::onMouseOn()
{
	setMode(GUI_MODE_ROLLOVER);
}

void GUIButton::onMouseOver()
{
	setMode(GUI_MODE_NORMAL);
}

void GUIButton::render()
{
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	if (active)
	{
		if ( phase <=0 )
		{
			bindTexture(textures[mode]);
			GuiRenderQuad(pos, size);		
		}
		else
		{
			glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
			bindTexture(textures[mode]);
			GuiRenderQuad(pos, size);

			glColor4f(1.0f, 1.0f, 1.0f, phase);
			bindTexture(textures[old_mode]);
			GuiRenderQuad(pos, size);
		}

	}
	else
	{
		glColor4f(1.0f, 1.0f, 1.0f, 0.45f);
		bindTexture(textures[GUI_MODE_DISABLE]);
		GuiRenderQuad(pos, size);
	}

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

void GUIButton::renderText()
{
	// 
	vec2 newTextPos;

	float letters_count = (float) text.getLength()+1.0f;	//      +1   :(
	float caption_width = letters_count * text_size * 0.5f;
	newTextPos.x = (size.x - caption_width)*0.5f;
	newTextPos.y = (size.y - text_size)*0.5f;


	if (!active)
	{
		glColor4f(1.0f, 1.0f, 1.0f, 0.45f);
		font.drawText(newTextPos + pos, text_size, text);
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	}
	else
		font.drawText(newTextPos + pos, text_size, text);
}






// --------------------------------------------------------------------
// GUICheckBox
//

void GUICheckBox::initialize(vec2& Pos, vec2& Size, float textSize)
{
	pos = Pos;
	size = Size;
	text_size = textSize;
	textPos = vec2(size.x, (size.y - text_size)*0.5f);
	checked = false;
	mode = GUI_MODE_UNCHECKED;
	old_mode = mode;
}

bool GUICheckBox::onCursor(vec2 p)
{
	float letters_count = (float) text.getLength()+1.0f;
	float caption_width = letters_count * text_size * 0.5f;

	return    (p.x > pos.x) && (p.x < pos.x + size.x+caption_width) &&
			  (p.y > pos.y) && (p.y < pos.y + size.y) && active;	
}

void GUICheckBox::onClick()
{
	if (active)
	{
		checked = !checked;
		if ( checked )
			setMode(GUI_MODE_CHECKED);
		else
			setMode(GUI_MODE_UNCHECKED);
	}
}

void GUICheckBox::onMouseOn()
{
	focused = true;
}

void GUICheckBox::onMouseOver()
{
	focused = false;
}

void GUICheckBox::render()
{
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	if (active)
	{
		vec2 s;
	    if ( focused )
			s = size*0.05f;
		else
			s = vec2(0.0f);

		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		bindTexture(textures[mode]);
		GuiRenderQuad(pos-s, size+s+s);

		glColor4f(1.0f, 1.0f, 1.0f, phase);
		bindTexture(textures[old_mode]);
		GuiRenderQuad(pos-s, size+s+s);
	}
	else
	{
		glColor4f(1.0f, 1.0f, 1.0f, 0.45f);
		bindTexture(textures[GUI_MODE_UNCHECKED]);
		GuiRenderQuad(pos, size);
	}

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

void GUICheckBox::renderText()
{
	if (!active)
	{
		glColor4f(1.0f, 1.0f, 1.0f, 0.45f);
		font.drawText(textPos + pos, text_size, text);
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	}
	else
		font.drawText(textPos + pos, text_size, text);
}






// --------------------------------------------------------------------
// GUIEdit
//

void GUIEdit::initialize(vec2& Pos, vec2& Size, float textSize)
{
	pos = Pos;
	size = Size;
	text_size = textSize;
	textPos = vec2(0.0f, (size.y - text_size)*0.5f);
	focused = false;
	onKeyPressEvent = NULL;
	maxSymbols = (int)(size.x/text_size*2.0f - 1.0f)+20;
	carretPos = 0;
	minVisibleCarretPos = 0;
	carretPhase = 0.0f;
}

void GUIEdit::render()
{
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	if (active)
	{
		vec2 s;
		if ( this->focused || GUIFrame::focused == this )
		{
			s = size*0.02f;
		}
		else
		{
			s = vec2(0.0f);
		}

		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		bindTexture(texture);
		GuiRenderQuad(pos-s, size+s+s);

		if ( GUIFrame::focused == this )
		{
			glColor4f(1.0f, 1.0f, 1.0f, clamp(cosf(carretPhase)*2.0f, 0.0f, 1.0f));
			font.drawText(pos + vec2((float)carretPos*text_size*0.5f-text_size*0.25f, textPos.y), text_size, "|");
		}
	}
	else
	{
		glColor4f(1.0f, 1.0f, 1.0f, 0.45f);
		bindTexture(texture);
		GuiRenderQuad(pos, size);
	}

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

void GUIEdit::renderText()
{
	if ( !active )
	{
		glColor4f(1.0f, 1.0f, 1.0f, 0.45f);
		font.drawText(textPos + pos, text_size, visibleText);
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	}
	else
		font.drawText(textPos + pos, text_size, visibleText);
}

void GUIEdit::update(float dt)
{
	carretPhase += dt*0.15f;	//  


	int symbolsInEdit = (int) (size.x/text_size*2.0f) - 1;

	if ( GUIFrame::focused == this )
	{

		//  
		if ( Key_Press(K_RIGHT) ) 
		{
			if ( carretPos < symbolsInEdit && carretPos + minVisibleCarretPos < text.getLength() )
			{
				carretPos++;
			}

			if ( carretPos >= symbolsInEdit )
			{
				if ( minVisibleCarretPos < text.getLength() - symbolsInEdit )
					minVisibleCarretPos++;

				carretPos = symbolsInEdit;
			}
		}
		else
			if ( Key_Press(K_LEFT)) 
			{
				if ( carretPos > 0 )
				{
					carretPos--;
				}

				if ( carretPos <= 0 )
				{
					if ( minVisibleCarretPos > 0 )
						minVisibleCarretPos--;

					carretPos = 0;
				}
			}
			else
				if (keyDown())
				{
					int lastKey = Key_LastPress();

					if ( onKeyPressEvent != NULL)
						onKeyPressEvent(lastKey);


					switch (lastKey)		//  
					{
					case K_BACKSPACE:		//   
						if ( carretPos > 0 )
						{
							text.remove(minVisibleCarretPos+carretPos-1, 1);
							carretPos--;
							if ( carretPos<=0 && minVisibleCarretPos>0) 
							{
								minVisibleCarretPos--;
								carretPos++;
							}
						}
						break;

					case K_DELETE:			//   
						if ( carretPos < text.getLength() )
							text.remove(minVisibleCarretPos+carretPos, 1);

						break;
										
					case K_HOME:			// 
						carretPos = 0;
						minVisibleCarretPos = 0;

						break;

					case K_END:				//  
						if ( text.getLength() > symbolsInEdit )
						{
							minVisibleCarretPos = text.getLength()-symbolsInEdit;
							carretPos = symbolsInEdit;
						}
						else
						{
							minVisibleCarretPos = 0;
							carretPos = text.getLength();
						}

						break;

					default:				//  
						tmpString = (unsigned char*)Key_EndReadText();
						Key_BeginReadText(NULL);
						if ( tmpString[0] > 31 )
							if ( text.getLength() < maxSymbols )
							{
								text.insert(minVisibleCarretPos+carretPos, (const char*) tmpString, 1);
								if ( carretPos < symbolsInEdit )
									carretPos++;
								else
									minVisibleCarretPos++;
							}

						break;
					}
				}
	}

	//   
	visibleText.assign(text.getStr() + minVisibleCarretPos, symbolsInEdit);
}

void GUIEdit::onClick()
{
	vec2 curPos = vec2((float)Mouse_X(), 0.0f);
	curPos.x -= pos.x;
	curPos.x /= (text_size*0.5f);
	carretPos = (int)curPos.x;

	if ( carretPos < 0) carretPos = 0;
	if ( carretPos > text.getLength() ) carretPos = text.getLength();
}

void GUIEdit::onMouseOn()
{
	focused = true;
}

void GUIEdit::onMouseOver()
{
	focused = false;
}





// --------------------------------------------------------------------
// GUIListBox
//

void GUIListBox::initialize(vec2& Pos, vec2& Size, float textSize)
{
	pos = Pos;
	size = Size;
	text_size = textSize;
	focused = false;
	selectedElement = -1;
}

void GUIListBox::addElement(const char* Text, void* Object)
{
	elements.add(new ListBoxElement(Text, Object));
}

void GUIListBox::insertElement(const char* Text, void* Object, unsigned int index)
{
	//////////////////////////////////////////////////////////////////
}

void GUIListBox::selectElement(unsigned int index)
{
	selectedElement = index;
	if ( selectedElement > elements.getCount()-1 ) selectedElement = elements.getCount()-1;
	if ( selectedElement < 0 ) selectedElement = 0;
	if ( elements.getCount() == 0 ) selectedElement = -1;
}

void GUIListBox::deleteElement(unsigned int index)
{
	if ( index > elements.getCount()-1 ) index = elements.getCount()-1;
	if ( index < 0 ) index = 0;
	if ( elements.getCount() != 0 )
	{
		void* del = elements[index];
		elements.orderedRemove(index);
		delete del;
	}
}

void GUIListBox::clearList()
{
	elements.clear();
}

void GUIListBox::render()
{
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	if (active)
	{
		vec2 s;
		if ( this->focused || GUIFrame::focused == this )
		{
			//s = size*0.02f;
			s = vec2(0.0f);
		}
		else
		{
			s = vec2(0.0f);
		}

		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		bindTexture(texture);
		GuiRenderQuad(pos-s, size+s+s);

		if ( selectedElement != -1 )
		{
			glDisable(GL_TEXTURE_2D);

			glColor4f(1.0f, 1.0f, 1.0f, 0.4f);

			GuiRenderQuad(vec2(10.0f + pos.x, 10.0f + pos.y+text_size*(float)selectedElement), vec2(size.x-10.0f, text_size));

			glEnable(GL_TEXTURE_2D);
		}
	}
	else
	{
		glColor4f(1.0f, 1.0f, 1.0f, 0.45f);
		bindTexture(texture);
		GuiRenderQuad(pos, size);
	}

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

void GUIListBox::renderText()
{
	for ( unsigned int i=0; i < elements.getCount(); i++ )
	{
		font.drawText(vec2(10.0f+pos.x, 10.0f + pos.y+text_size*(float)i), 25.0f, elements[i]->text);
	}
}

void GUIListBox::update(float dt)
{

}

void GUIListBox::onClick()
{
	vec2 curPos = vec2(0.0f, (float)Mouse_Y()-10.0f);
	curPos.y -= pos.y;
	curPos.y /= (text_size);

	selectedElement = (int)curPos.y;
	if ( selectedElement > elements.getCount()-1 ) selectedElement = elements.getCount()-1;
	if ( selectedElement < 0 ) selectedElement = 0;
	if ( elements.getCount() == 0 ) selectedElement = -1;
}

void GUIListBox::onMouseOn()
{
	focused = true;
}

void GUIListBox::onMouseOver()
{
	focused = false;
}




// --------------------------------------------------------------------
// GUIFrame
//

int GUIFrame::addElement(GUIElement* elem)
{
	elements.add(elem);
	return elements.getCount()-1;
}

void GUIFrame::deleteElement(int index)
{
	GUIElement* f;
	f = elements[index];
	delete f;
	elements.fastRemove(index);
}

void GUIFrame::render()
{
	for (unsigned int i = 0; i < elements.getCount(); i++)
		{
			elements[i]->render();
			elements[i]->renderText();
		}
}

void GUIFrame::update(float dt)
{
	for (int i = elements.getCount()-1; i >= 0 ; i--)
	{
		elements[i]->update(dt);
		if (elements[i]->onCursor(vec2((float)Mouse_X(), (float)Mouse_Y())) && elements[i]->active)
		{
			elements[i]->onMouseOn();
			if (Mouse_Click(M_LBUTTON))
			{
				elements[i]->onClick();
				focused = elements[i];
			}
		}
		else
		{
			elements[i]->onMouseOver();
		}
	}
}




// --------------------------------------------------------------------
// GUIEngine
//

int GUIEngine::addWindow(GUIFrame* elem)
{
	frames.add(elem);
	return frames.getCount()-1;
}

void GUIEngine::deleteWindow(int index)
{
	GUIFrame* f;
	f = frames[index];
	delete f;
	frames.fastRemove(index);
}

void GUIEngine::setCurrentWindow(int index)
{
	activeFrame = index;
}

void GUIEngine::setCurrentWindow(GUIFrame* wind)
{
	activeFrame = frames.getIndex(wind);
}

void GUIEngine::update(float dt)
{
	if ( activeFrame >=0 && activeFrame < frames.getCount() )
		frames[activeFrame]->update(dt);
}

void GUIEngine::render()
{
	if ( activeFrame >=0 && activeFrame < frames.getCount() )
		frames[activeFrame]->render();
}




