//
//     
// :      Framework'
//
//  - ViZgl
//



#ifndef __GUI_h__
#define __GUI_h__

#include <windows.h>
#include <gl/gl.h>
#include "Array.h"
#include "Font.h"
#include "String.h"






//
//  
//

class GUIElement;
class GUIPanel;
class GUILabel;
class GUIButton;
class GUICheckBox;
class GUIEdit;

class GUIListBox;

class GUIFrame;
class GUIEngine;






//
// 
//

typedef void (* POnClickEvent)();			//  
typedef void (* POnKeyPressEvent)(int key);	//  






//
// 
//

#define GUI_MODE_NORMAL					0	//  !!!!!     !!!!
#define GUI_MODE_ROLLOVER				1
#define GUI_MODE_DISABLE				2

#define GUI_MODE_UNCHECKED				0
#define GUI_MODE_CHECKED				1






//
// GUIElement
//

class GUIElement
{
protected:
	vec2   pos;			// 
	vec2   size;		// 
	bool   active;      // ?
	int    mode;		//  
	int    old_mode;	//     
	float  phase;		//    
	String text;		//   

	float text_size;


	vec2   textPos;		//    
	float  fadeScale;	//  


	virtual bool onCursor(vec2 p);
    virtual void setMode(int m);
    virtual void onClick();
    virtual void onMouseOn();
    virtual void onMouseOver();
    virtual void render();
    virtual void renderText();
	virtual void update(float dt);

public:
	GUIElement();

	// set/get
	void setPosition(const vec2& Pos) { pos = Pos; }
	vec2 getPosition() const { return pos; }

	void setSize(const vec2& Size) { size = Size; }
	vec2 getSize() const { return size; }

	void setActive(const bool state) { active = state; }
	bool getActive() { return active; }

	void setText(const String& Text) { text = Text; }
	const String& getText() const { return text; }

	void setBlinkTime(const float coeff) { fadeScale = coeff; }
	float getBlinkTime() const { return fadeScale; }

	void setTextSize(const float TextSize) { text_size = TextSize; }
	float getTextSize() const { return text_size; }


	friend GUIFrame;
};







//
// GUIPanel
//

class GUIPanel: public GUIElement
{
protected:
    unsigned int texture;
	float alpha;

	void render();

public:
	void initialize(vec2& Pos, vec2& Size, float Alpha);

	// set/get
	void setTexture(const unsigned int NewTexture) { texture = NewTexture; }
	unsigned int getTexture() const { return texture; }

	void setAlpha(const float Alpha) { alpha = Alpha; }
	float getAlpha() { return alpha; }

};







//
// GUILabel
//

class GUILabel: public GUIElement
{
	void renderText();

public:
	void initialize(vec2& Pos, float TextSize);
    
};







//
// GUIButton
//

class GUIButton: public GUIElement
{
protected:
	unsigned int textures[3];	// 1 - normal
								// 2 - rollover
								// 3 - disabled

    void onClick();
    void onMouseOn();
    void onMouseOver();
	void render();
    void renderText();

public:
	void initialize(vec2& Pos, vec2& Size, float textSize=25.0f);

	void setTextures(const unsigned int TextureNormal,
		const unsigned int TextureRollOver,
		const unsigned int TextureDisabled) { textures[0] = TextureNormal; textures[1] = TextureRollOver; textures[2] = TextureDisabled; }
	unsigned int getTextureNormal()   const { return textures[0]; }
	unsigned int getTextureRollOver() const { return textures[1]; }
	unsigned int getTextureDisabled() const { return textures[2]; }


	POnClickEvent OnClickEvent;
	
};






//
// GUICheckBox
//

class GUICheckBox: public GUIElement
{
public:
	void initialize(vec2& Pos, vec2& Size, float textSize=25.0f);

	bool getChecked() { return checked; };
	void setChecked(bool Checked) {	checked = Checked; if ( checked ) setMode(GUI_MODE_CHECKED); else setMode(GUI_MODE_UNCHECKED); }

	void setTextures(const unsigned int TextureUnCkecked,
		const unsigned int TextureCkecked)   { textures[0] = TextureUnCkecked; textures[1] = TextureCkecked; }
	unsigned int getTextureUnCkecked() const { return textures[0]; }
	unsigned int getTextureCkecked() const   { return textures[1]; }


protected:
	bool checked;
	bool focused;
	int  textures[2];	// 1 - unchecked
						// 2 - checked

	bool onCursor(vec2 p);
    void onClick();
    void onMouseOn();
    void onMouseOver();
	void render();
    void renderText();
};






//
// GUIEdit
//

class GUIEdit: public GUIElement
{
public:
	void initialize(vec2& Pos, vec2& Size, float textSize=25.0f);

	// set/get
	void setMaxSymbols(const int MaxSymbols) { maxSymbols = MaxSymbols; }
	unsigned int getMaxSymbols() const { return maxSymbols; }
	void setTextSize(const float TextSize) { text_size = TextSize; }
	float getTextSize() { return text_size; }
	void setTexture(const unsigned int Texture) { texture = Texture; }
	unsigned int getTexture() { return texture; }
	

	POnKeyPressEvent onKeyPressEvent;

protected:
	bool focused;
	String visibleText;					//     
	float carretPhase;					//  
	int   carretPos;					//     
	int   minVisibleCarretPos;			//     


	// set/get
	unsigned int maxSymbols;
	float text_size;
	unsigned int   texture;


	void render();
    void renderText();
	void update(float dt);
    void onClick();
    void onMouseOn();
    void onMouseOver();
};





//
// GUIListBox
//

struct ListBoxElement
{
public:
	String text;
	void*  object;

	ListBoxElement(const char* Text, void* Object)
	{
		text = Text;
		object = Object;
	}
};

class GUIListBox: public GUIElement
{
public:
	void initialize(vec2& Pos, vec2& Size, float textSize=25.0f);

	// set/get
	void setTextSize(const float TextSize) { text_size = TextSize; }
	float getTextSize() { return text_size; }
	void setTexture(const unsigned int Texture) { texture = Texture; }
	unsigned int getTexture() { return texture; }

	void addElement(const char* Text, void* Object);
	void insertElement(const char* Text, void* Object, unsigned int index);
	void selectElement(unsigned int index);
	void deleteElement(unsigned int index);
	void clearList();
	
protected:

	Array<ListBoxElement*> elements;		//  
	bool focused;
	int   selectedElement;					//  

	unsigned int firstVisible;


	// set/get
	float text_size;
	unsigned int texture;


	void render();
    void renderText();
	void update(float dt);
    void onClick();
    void onMouseOn();
    void onMouseOver();
};





//
// GUIFrame
//

class GUIFrame
{
public:
	Array<GUIElement*>	elements;
	bool active;

	static GUIElement*  focused;			//     

public:
	~GUIFrame () { clear (); }
	int addElement(GUIElement* elem);
	void deleteElement(int index);
	void render();
	void update(float dt);
	void clear ()
	{
		for ( unsigned int i = 0; i < elements.getCount (); i++ )
		{
			free ( elements [i] );
		}
		elements.clear ();
	}
};






//
// GUIEngine
//


class GUIEngine
{
public:
    Array<GUIFrame*>   frames;
	int				   activeFrame;

public:
	~GUIEngine () { clear (); }
	int  addWindow(GUIFrame* elem);
	void deleteWindow(int index);
	void setCurrentWindow(int index);
	void setCurrentWindow(GUIFrame* wind);
    void render();
	void update(float dt);
	void clear ()
	{
		for ( unsigned int i = 0; i < frames.getCount (); i++ )
		{
			free ( frames [i] );
		}
		frames.clear ();
		activeFrame = -1;
	}
};





bool keyDown();								//  ,   1 



//
//   
//

extern GUIEngine gui;



#endif




