#include "SpriteAnimation.h"

Animation::Animation()
{}

void Animation::init( vec2& size, //   
				  int tex,	  // 
				  int tex_w,  //  
				  int tex_h,  //  
				  int nFrames,//   
				  float fps,  //    
				  int sprite_width,
				  int sprite_height,
				  bool Loop,
				  bool Active,
				  bool Alpha)
{
	loop = Loop;
	active = Active;
	alpha = Alpha;
	this->size= size;
	half_size = size * 0.5f;

    this->sprite_width = sprite_width;
    this->sprite_height= sprite_height;

	texture = tex;
	tex_width = tex_w;
	tex_height= tex_h;
	FPS = 1.0f / fps;	
	framesCount = nFrames;
	current_frame = 1;

	timer = 0.0f;


	double _width = 1.0/tex_width;		//  
	double _height= 1.0/tex_height;     //  

	width_t = sprite_width * _width;	//     
	height_t= sprite_height * _height;  

	num_in_row = tex_width / sprite_width;  //    
	num_in_col = tex_height/ sprite_height; //    


	setFrame(1);
}


void Animation::update(float dt)
{
	if (active)
	{
		timer += dt*FPS;
		if (timer > 1.0f) { current_frame++; timer = 0.0f; }
		if (!loop && current_frame>framesCount) { active = false; return; }
		if (current_frame > framesCount && loop) {current_frame = 1;}
		setFrame(current_frame);
	}
}


void Animation::render( vec2 & pos, float angle  )
{
	if (!alpha) glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	else   	    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	if (active)
	{
		glPushMatrix ();
		{
			glTranslatef ( pos.x, pos.y, 0.0f );
			glRotatef ( angle, 0.0f, 0.0f, 1.0f );

			glBindTexture ( GL_TEXTURE_2D, texture );
			glBegin(GL_QUADS);
			glTexCoord2f(current_u, current_v);						 glVertex2f(- half_size.x, - half_size.x);
			glTexCoord2f(current_u + width_t, current_v);			 glVertex2f(+ half_size.x, - half_size.x);
			glTexCoord2f(current_u + width_t, current_v - height_t); glVertex2f(+ half_size.x, + half_size.x);
			glTexCoord2f(current_u, current_v - height_t);			 glVertex2f(- half_size.x, + half_size.x);
			glEnd();
		}
		glPopMatrix ();
	}
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}


void Animation::setFrame(int n)
{
	if ( n < 1 )		   n = 1;
	if ( n > framesCount ) n = framesCount;  //    

	int field_j = (n-1) % num_in_row;  // x
	int field_i = (n-1) / num_in_row;  // y

	current_u = field_j * width_t;
	current_v = 1.0f - field_i * height_t;
}


void Animation::setActive(bool state)
{
	timer = 0.0f;
	active = state;
	current_frame = 1;
}


void Animation::setLoop(bool state)
{
	loop = state;
}

