#ifndef _TOKENIZER_H_
#define _TOKENIZER_H_

#include "Array.h"

typedef bool ( *BOOLFUNC ) ( const char ch );

bool isWhiteSpace ( const char ch );
bool isNumeric ( const char ch );
bool isAlphabetical ( const char ch );
bool isNewLine ( const char ch );

struct TokBuffer
{
	char *buffer;
	unsigned int bufferSize;
};


class Tokenizer
{
public:
	Tokenizer ( unsigned int nBuffers = 1 );
	~Tokenizer ();

	void setString ( const char *string );
	bool setFile ( const char *fileName );
	void reset ();

	bool goToNext ( BOOLFUNC isAlpha = isAlphabetical );
	bool goToNextLine ();
	char *next ( BOOLFUNC isAlpha = isAlphabetical );
	char *nextAfterToken ( const char *token, BOOLFUNC isAlpha = isAlphabetical );
	char *nextLine ();

private:
	char *str;
	unsigned int length;
	unsigned int start, end;

	unsigned int currentBuffer;
	Array <TokBuffer> buffers;

	char *getBuffer ( unsigned int size );
};

#endif
