#ifndef __TEXTURE__
#define __TEXTURE__

#include    <windows.h>
#include    <GL/gl.h>
#include    <GL/glu.h>

class Texture
{
protected:
    int				width;
    int				height;
    int				numComponents;
    int				format;
    unsigned char * data;
    int				levels;
    bool			compressed;

public:
    Texture ()
	{
		width         = 0;
		height        = 0;
		numComponents = 0;
		data          = NULL;
		levels        = 0;
		compressed    = false;
		format        = GL_NONE;
	}

    Texture ( int theWidth, int theHeight, int theNumComponents )
	{
		width         = theWidth;
		height        = theHeight;
		numComponents = theNumComponents;
		data          = (unsigned char *) malloc ( width * height * numComponents );
		levels        = 1;
		compressed    = false;

		switch ( numComponents )
		{
			case 1:
				format = GL_ALPHA;
				break;

			case 3:
				format = GL_RGB;
				break;

			case 4:
				format = GL_RGBA;
				break;

			default:
				format = -1;
		}
	}

    virtual ~Texture ()
	{
		if ( data != NULL )
			free ( data );
	}

    int getWidth () const
    {
        return width;
    }

    int getHeight () const
    {
        return height;
    }

    int getNumComponents () const
    {
        return numComponents;
    }

    int getFormat () const
    {
        return format;
    }

    int getLevels () const
    {
        return levels;
    }

    bool isCompressed () const
    {
        return compressed;
    }

    int getBytesPerLine () const
    {
        return width * numComponents;
    }

    unsigned char * getData () const
    {
        return data;
    }

    void setFormat ( int newFormat )
    {
        format = newFormat;
    }

    void putLine ( int y, unsigned long * bits )
	{
		if ( y < 0 || y >= height )
			return;

		int offs = y * width * numComponents;
		unsigned char * ptr  = data + offs;

		if ( numComponents == 4 )               // RGBA image
			memcpy ( ptr, bits, 4 * width );
		else
		if ( numComponents == 3 )               // RGB image
		{
			unsigned char * src = (unsigned char *) bits;

			for ( int i = 0; i < width; i++, src += 4 )
			{
				*ptr++ = src [0];
				*ptr++ = src [1];
				*ptr++ = src [2];
			}
		}
		else
		if ( numComponents == 1 )               // greyscale image
		{
			for ( int i = 0; i < width ; i++, bits++ )
				*ptr++ = *(unsigned char *) bits;
		}
	}

											//     
    virtual bool upload ( int target, bool mipmap = true )
	{
		if ( target == GL_TEXTURE_1D )
		{
			if ( mipmap )
				gluBuild1DMipmaps ( target, getNumComponents (), getWidth (),
									getFormat (), GL_UNSIGNED_BYTE, getData () );
			else
				glTexImage1D      ( target, 0, getNumComponents (), getWidth (), 0,
									getFormat (), GL_UNSIGNED_BYTE, getData () );
		}
		else
		{
			if ( mipmap )
				gluBuild2DMipmaps ( target, getNumComponents (), getWidth (), getHeight (),
									getFormat (), GL_UNSIGNED_BYTE, getData () );
			else
				glTexImage2D      ( target, 0, getNumComponents (), getWidth (), getHeight (), 0,
									getFormat (), GL_UNSIGNED_BYTE, getData () );
		}

		return true;
	}
};

inline unsigned long makeRgba ( int red, int green, int blue, int alpha = 255 )
{
    return red + (green << 8) + (blue << 16) + (alpha << 24);
}

#endif
