#include "TextureManager.h"

#include "Texture.h"
#include "TextureLoader.h"
#include "PngLoader.h"

#include "../Utils/Data.h"

#pragma warning (disable:4996)
#pragma	warning (disable:4786)


void TextureManager :: initManager ()
{
	static bool inited = false;

	if ( inited )
		return;

	addDecoder ( "png",  new PngLoader );

	inited = true;
}

bool TextureManager :: addDecoder ( const char * ext, TextureLoader * decoder )
{
	char extension [128];

	if ( strlen ( ext ) >= 128 )
		return false;

	strlwr ( strcpy ( extension, ext ) );

	map <String, TextureLoader *> :: iterator it = decoders.find ( extension );

	if ( it != decoders.end () )
		return false;

	decoders [extension] = decoder;

	return true;
}

TextureInfo* TextureManager :: findTexture ( unsigned int ID, unsigned int &index )
{
	for ( unsigned int i = 0; i < cache.getCount (); i++ )
	{
		if ( cache[i]->id == ID )
		{
			index = i;
			return cache[i];
		}
	}

	return NULL;
}

TextureInfo* TextureManager :: findTexture ( const String &fileName, unsigned int &index )
{
	for ( unsigned int i = 0; i < cache.getCount (); i++ )
	{
		if ( cache[i]->fileName == fileName )
		{
			index = i;
			return cache[i];
		}
	}

	return NULL;
}

bool TextureManager :: deleteTexture ( unsigned int ID )
{
	unsigned int index;
	TextureInfo *t = findTexture ( ID, index );

	if ( t == NULL )
		return false;

	if ( t->refCount > 1 )
	{
		t->refCount --;
		return true;
	}

	glDeleteTextures ( 1, &t->id );
	cache.fastRemove ( index );

	delete t;

	return true;
}

Texture* TextureManager :: getTexture ( const char * fileName )
{
	initManager ();								// it's safe to place it anywhere

    const char * ext = strrchr ( fileName, '.' );

    if ( ext == NULL || strlen ( ext ) >= 128 )
        return NULL;

	char	  extension [128];
    Data    * data = new Data ( fileName );
    Texture * tex = NULL;

    if ( data == NULL || data -> getLength () < 1 )
    {
        fprintf ( stderr, "Cannot load \"%s\"\n", fileName );

        return NULL;
    }

	strlwr ( strcpy ( extension, ext + 1 ) );

	map <String, TextureLoader *> :: iterator it = decoders.find ( extension );

	if ( it != decoders.end () )
		tex = it -> second -> load ( data );

    if ( tex == NULL )
        fprintf ( stderr, "No loader for %s !!!\n", fileName );

    delete data;

    return tex;
}

bool TextureManager :: loadTexture ( int target, bool mipmap, const char * fileName )
{
    Texture * texture = getTexture ( fileName );

    if ( texture == NULL )
        return false;

    texture -> upload ( target, mipmap );

    if ( mipmap )
        glTexParameteri ( target, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
    else
        glTexParameteri ( target, GL_TEXTURE_MIN_FILTER, GL_LINEAR );

    delete texture;

    return true;
}

unsigned int TextureManager :: createTexture2D ( bool mipmap, const char * fileName )
{
																		// try to search in cache
	unsigned int index;
	TextureInfo *info =  findTexture ( fileName, index );

	if ( info != NULL )
	{
		info->refCount ++;
		return info->id;
	}

    GLuint    textureId;

    glGenTextures   ( 1, &textureId );
    glBindTexture   ( GL_TEXTURE_2D, textureId );
    glPixelStorei   ( GL_UNPACK_ALIGNMENT, 1 );                         // set 1-byte alignment

    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );    // set texture to repeat mode
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );

    if ( !loadTexture ( GL_TEXTURE_2D, mipmap, fileName ) )
        return 0;

    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

																		// add texture to cache
	TextureInfo *newTexture = new TextureInfo ();

	newTexture->fileName = fileName;
	newTexture->id = textureId;
	newTexture->refCount = 1;

	cache.add ( newTexture );

    return textureId;
}

unsigned int TextureManager :: createTexture1D ( bool mipmap, const char * fileName )
{
																		// try to search in cache
	unsigned int index;
	TextureInfo *info =  findTexture ( fileName, index );

	if ( info != NULL )
	{
		info->refCount ++;
		return info->id;
	}

    GLuint    textureId;

    glGenTextures   ( 1, &textureId );
    glBindTexture   ( GL_TEXTURE_1D, textureId );
    glPixelStorei   ( GL_UNPACK_ALIGNMENT, 1 );                         // set 1-byte alignment

    glTexParameteri ( GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, GL_REPEAT );    // set texture to repeat mode
    glTexParameteri ( GL_TEXTURE_1D, GL_TEXTURE_WRAP_T, GL_REPEAT );

    if ( !loadTexture ( GL_TEXTURE_1D, mipmap, fileName ) )
        return 0;

    glTexParameteri ( GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

																		// add texture to cache
	TextureInfo *newTexture = new TextureInfo ();

	newTexture->fileName = fileName;
	newTexture->id = textureId;
	newTexture->refCount = 1;

	cache.add ( newTexture );

    return textureId;
}
