unit raEnemies;

interface
  uses OMEGA, Helper, raBullets;

type
  TEnemyStage = (esGrow, esUnGrow, es_Active, esDie);

  TCircleUFO = record
  private
    X, Y: single;
    //VelX, VelY: single;
    //MaxVel: single;
    TX, TY: single;
    Live: boolean;
    Stage: TEnemyStage;
    Size, Size2: single;
    FireTimer: single;
    FireTimes: integer;
  public
    procedure Create(PosX, PosY: single);
    procedure SetTarget(TaX, TaY: single);
    procedure Draw;
    procedure Update;
    procedure Fire;
    procedure Kill;
  end;

  TCircleUFO2 = record
  end;

  TRocketCenter = record
  private
    X, Y: single;
    Live: boolean;
    Stage: TEnemyStage;
    Size, Size2: single;
    FireTimer: single;
    target_player: boolean;
    kind: integer;
    FireTimes: integer;
  public
    procedure Create(PosX, PosY: single; RocketKind: integer);
    procedure SetTarget(player: boolean = true);
    procedure Draw;
    procedure Update;
    procedure Fire;
    function Die: boolean;
    function GetPos: oePoint;
    procedure Kill;
  end;

  TRocketCenter2 = record
  end;

var
  rcenters: array of TRocketCenter;

implementation
  uses raPlayer;

{ TCircleUFO }

procedure TCircleUFO.Create(PosX, PosY: single);
begin
  X := PosX;
  Y := PosY;
  //VelX := 0;
  //VelY := 0;
  //MaxVel := 5;
  live := true;
  Stage := esGrow;
  Size := 0;
  FireTimes := 0;
end;

procedure TCircleUFO.Draw;
begin
  if live then
  begin
    Pr2D_Circle(X, Y, Size2, OmegaColor(255, 255, 255), 255, 32, PR_FILL);
    Pr2D_Circle(X, Y, Size2, OmegaColor(0, 0, 0), 255, 32, PR_SMOOTH);
    Pr2D_Circle(X, Y, Size, OmegaColor(255, 255, 128), 255, 32, PR_FILL);
  end;
end;

procedure TCircleUFO.Fire;
begin
  Inc(FireTimes);
  CreateBullet(X, Y, TX, TY, 4);
end;

procedure TCircleUFO.Kill;
begin
  Stage := esDie;
end;

procedure TCircleUFO.SetTarget(TaX, TaY: single);
begin
  TX := TaX;
  TY := TaY;
end;

procedure TCircleUFO.Update;
begin
  if live then
  begin
    //VelX := GetSpeedX(X, Y, TX, TY);
    //VelY := GetSpeedY(X, Y, TX, TY);

    //X := X + VelX;
    //Y := Y + VelY;

    case Stage of
      esGrow: 
        if Size < 20 then
          Size := Size + 1
        else
        begin
          Stage := esUnGrow;
          Size2 := 16;
        end;
      esUnGrow:
        if Size > 8 then
          Size := Size - 0.5
        else
          Stage := es_Active;
      es_Active: begin
        if U_GetTime - FireTimer > 1000 then
        begin
          Fire;
          FireTimer := U_GetTime;
        end;
        if FireTimes > 10 then
          Stage := esDie;
      end;
      esDie:
        if Size2 > Size then
          Size2 := Size2 - 1
        else
        begin
          Size2 := 0;
          if Size > 0 then
            Size := Size - 1
          else
             Live := false;
        end;
    end;
  end;
end;

{ TRocketCenter }

procedure TRocketCenter.Create;
begin
  X := PosX;
  Y := PosY;
  live := true;
  Stage := esGrow;
  Size := 0;
  target_player := true;
  kind := RocketKind;
  FireTimes := 0;
end;

function TRocketCenter.Die: boolean;
begin
  Result := false;
  if not Live then
    Result := true;
end;

procedure TRocketCenter.Draw;
var
  color: integer;
begin
  if live then
  begin
    case kind of
      0: color := OmegaColor(255, 128, 128);
      1: color := OmegaColor(128, 128, 255);
    end;
    Pr2D_Circle(X, Y, Size2, OmegaColor(255, 255, 255), 255, 32, PR_FILL);
    Pr2D_Circle(X, Y, Size2, OmegaColor(0, 0, 0), 255, 32, PR_SMOOTH);
    Pr2D_Circle(X, Y, Size, color, 255, 32, PR_FILL);
  end;
end;

procedure TRocketCenter.Fire;
begin
  Inc(FireTimes);
  //CreateRocket(X, Y, 0, target_player);
  CreateNewRocket(X, Y, M_FindAngle(Round(player.GetPos.X), Round(player.GetPos.Y), Round(X), Round(Y)), kind, target_player);
end;

function TRocketCenter.GetPos: oePoint;
begin
  Result.X := X;
  Result.Y := Y;
end;

procedure TRocketCenter.Kill;
begin
  Stage := esDie;
end;

procedure TRocketCenter.SetTarget(player: boolean);
begin
  target_player := player;  
end;

procedure TRocketCenter.Update;
begin
  if live then
  begin
    case Stage of
      esGrow: 
        if Size < 40 then
          Size := Size + 1
        else
        begin
          Stage := esUnGrow;
          Size2 := 32;
        end;
      esUnGrow:
        if Size > 16 then
          Size := Size - 0.5
        else
        begin
          Stage := es_Active;
          FireTimer := U_GetTime - 8000;
        end;
      es_Active: begin
        if (U_GetTime - FireTimer > 10000) or (length(nrs) < 1) then
        begin
          Fire;
          FireTimer := U_GetTime;
        end;
      end;
      esDie:
        if Size2 > Size then
          Size2 := Size2 - 1
        else
        begin
          Size2 := 0;
          if Size > 0 then
            Size := Size - 1
          else
             Live := false;
        end;
    end;
  end;
end;

end.
