////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////     01010101   010        010   0101010101   0101010101      0110      ////
////    01      01  01010    10101   01           01             01  10     ////
////   01        01 01 010  101 01   01           01             01  10     ////
////   01        01 01  010010  01   0101010101   01            01    10    ////
////    01      01  01    00    01   01           01    0101    01010110    ////
////     01    01   01          01   01           01      01   01      10   ////
////   0101    0101 01          01   0101010101   0101010101   01      10   ////
////                                                                        ////
////                                                                        ////
////                    0101010101   01     01   010101                     ////
////                    01           01     01     01                       ////
////                    01           01     01     01                       ////
////                    01           01     01     01                       ////
////                    01    0101   01    011     01                       ////
////                    01      01   01  01011     01                       ////
////                    0101010101   01010  01   010101                     ////
////                                                                        ////
////       : WIP                                                      ////
////         : 04.10.07                                                 ////
////   e: http://www.isada.mirgames.ru/omega.php                   ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////  : Andru 'aka'                                         ////
////  Mail: Andru_@list.ru                                                  ////
////  Site: http://isada.mirgames.ru                                        ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////                                    :             ////
////                                                                        ////
////                ////
////      .      .             ////
////                                                             ()2007    ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
unit uGUIEngine;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ENDIF}

interface
uses
  OMEGA, uSpriteEngine;

type
  TGUIMouse = object
    Texture : PTexture;
    X, Y    : Integer;
    Pressed : Boolean; // Left Button

    procedure Process;
  end;

type
  TGUIFont = record
    fType : PFont2D;
    Scale : Single;
    Step  : Single;
    Alpha : Byte;
    Color : Integer;
end;

type
  TGUIColor = record
    Border   : Integer;
    Disabled : Integer;
    Focused  : Integer; // Border Color Change
    MouseIn  : Integer;
    Pressed  : Integer;
end;

type
  CGUIEngine = class( CSpriteEngine )
  protected
    CamNow : ^PCamera2D;
  public
    constructor Create;

    procedure Draw;    override;
    procedure Process; override;

    procedure ResetFocus( Parent : CSprite );

    function  AddElement( Element : CSprite ) : Integer;
    procedure DelElement( Element : CSprite );
end;

type
  CGUIElement = class( CSprite )
  protected
    KeyLastDown : Integer;
    KeyLastUp   : Integer;
  public
    Engine : CGUIEngine;
    Parent : CGUIElement;

    Font : TGUIFont;

    Caption : String;
    Enabled : Boolean;
    Visible : Boolean;
    Focused : Boolean;
    MouseIn : Boolean;

    Color  : Integer;
    AColor : TGUIColor;

    // Events
    POnPaint      : procedure( Sender, Parent : CGUIElement );
    POnShow       : procedure( Sender, Parent : CGUIElement );
    POnKeyDown    : procedure( KeyCode : Integer; Sender, Parent : CGUIElement );
    POnKeyUp      : procedure( KeyCode : Integer; Sender, Parent : CGUIElement );
    POnMouseMove  : procedure( X, Y : Single; Sender, Parent : CGUIElement );
    POnMouseClick : procedure( mButton : Integer; Sender, Parent : CGUIElement );
    POnMouseDown  : procedure( mButton : Integer; Sender, Parent : CGUIElement );
    POnMouseUp    : procedure( mButton : Integer; Sender, Parent : CGUIElement );
    POnProcess    : procedure( Sender, Parent : CGUIElement );

    constructor Create; override;

    procedure OnPaint;
    procedure OnShow;
    procedure OnKeyDown;
    procedure OnKeyUp;
    procedure OnMouseMove;
    procedure OnMouseClick;
    procedure OnMouseDown;
    procedure OnMouseUp;

    procedure Draw; override;
    procedure Process; override;
    procedure Show;
end;

type
  CGUIImage = class( CGUIElement )
  protected
  public
    AutoSize : Boolean;

    procedure Draw; override;
    procedure Process; override;
end;

type
  CGUILabel = class( CGUIElement )
  protected
  public

    procedure Draw; override;
    procedure Process; override;
end;

// 1 - Normal
// 2 - "MouseIn"
// 3 - "MouseDown"
// 4 - Disabled
type
  CGUIButton = class( CGUIElement )
  protected
  public

    procedure Draw; override;
    procedure Process; override;
end;

// 1 - Zero
// 2 - Checked
// 3 - "MouseIn" Zero
// 4 - "MouseIn" Checked
// 5 - Disabled
type
  CGUICheckBox = class( CGUIElement )
  protected
  public
    Checked : Boolean;

    POnChange : procedure( Checked : Boolean; Sender, Parent : CGUIElement );

    procedure OnChange;

    procedure Draw; override;
    procedure Process; override;
end;

type
  CGUIEdit = class( CGUIElement )
  protected
    LineTime  : Single;
    LineAlpha : Byte;
  public
    Max : Integer;

    procedure Draw; override;
    procedure Process; override;
end;

// 0 - Border
// 1 - Progress-line
type
  CGUIProgressBar = class( CGUIElement )
  protected
  public
    Max      : Integer;
    Progress : Integer;
    Style    : Byte;
    
    POnChange : procedure( Progress : Integer; Sender, Parent : CGUIElement );
    
    procedure OnChange;

    procedure Draw; override;
    procedure Process; override;
end;

// Utils
function CreateImage( Parent              : CGUIElement;
                      Texture             : PTexture;
                      X, Y, Width, Height : Single;
                      AutoSize            : Boolean = TRUE ) : CGUIImage;

function CreateLabel( Parent  : CGUIElement;
                      Font    : PFont2D;
                      X, Y    : Single;
                      Caption : String ) : CGUILabel;

function CreateButton( Parent              : CGUIElement;
                       Texture             : PTexture;
                       Font                : PFont2D;
                       Caption             : String;
                       X, Y, Width, Height : Single;
                       OnMouseClick : Pointer ) : CGUIButton;

function CreateCheckBox( Parent              : CGUIElement;
                         Texture             : PTexture;
                         Font                : PFont2D;
                         X, Y, Width, Height : Single;
                         Caption             : String;
                         Checked             : Boolean;
                         OnChange            : Pointer ) : CGUICheckBox;

function CreateEdit( Parent              : CGUIElement;
                     Texture             : PTexture;
                     Font                : PFont2D;
                     X, Y, Width, Height : Single;
                     Caption             : String;
                     Max                 : Integer = 20 ) : CGUIEdit;
                     
function CreateProgressBar( Parent              : CGUIElement;
                            Texture             : PTexture;
                            Font                : PFont2D;
                            X, Y, Width, Height : Single;
                            Color               : Integer = $FF0000;
                            Max                 : Integer = 100 ) : CGUIProgressBar;

function PointInRect( pX, pY, X, Y, Width, Height : Single ) : Boolean;

var
  Mouse : TGUIMouse;

  GUI   : CGUIEngine;

implementation

var
  LastZ        : DWORD = 0;
  GlobalParent : CGUIElement;

function CreateImage;
Begin
INC( LastZ );
Result          := CGUIImage.Create;
if Parent = nil Then
  Result.Parent := GlobalParent
else
  Result.Parent := Parent;
Result.Z        := LastZ;

Result.Texture  := Texture;
Result.AutoSize := AutoSize;
Result.X        := X;
Result.Y        := Y;

if AutoSize Then
  begin
    Result.Width    := Texture.Width;
    Result.Height   := Texture.Height;
  end else
    begin
      Result.Width  := Width;
      Result.Height := Height;
    end;
End;

function CreateLabel;
Begin
INC( LastZ );
Result          := CGUILabel.Create;
if Parent = nil Then
  Result.Parent := GlobalParent
else
  Result.Parent := Parent;
Result.Z        := LastZ;

Result.Texture    := nil;
Result.Font.fType := Font;
Result.X          := X;
Result.Y          := Y;
Result.Caption    := Caption;
End;

function CreateButton;
Begin
INC( LastZ );
Result          := CGUIButton.Create;
if Parent = nil Then
  Result.Parent := GlobalParent
else
  Result.Parent := Parent;
Result.Z        := LastZ;

Result.Texture       := Texture;
Result.Font.fType    := Font;
Result.Caption       := Caption;
Result.X             := X;
Result.Y             := Y;
Result.Width         := Width;
Result.Height        := Height;
Result.POnMouseClick := OnMouseClick;
End;

function CreateCheckBox;
Begin
INC( LastZ );
Result          := CGUICheckBox.Create;
if Parent = nil Then
  Result.Parent := GlobalParent
else
  Result.Parent := Parent;
Result.Z        := LastZ;

Result.Texture    := Texture;
Result.Font.fType := Font;
Result.Caption    := Caption;
Result.X          := X;
Result.Y          := Y;
Result.Width      := Width;
Result.Height     := Height;
Result.Color      := $FFFFFF;
End;

function CreateEdit;
Begin
INC( LastZ );
Result          := CGUIEdit.Create;
if Parent = nil Then
  Result.Parent := GlobalParent
else
  Result.Parent := Parent;
Result.Z        := LastZ;

Result.Texture    := Texture;
Result.Font.fType := Font;
Result.Caption    := Caption;
Result.X          := X;
Result.Y          := Y;
Result.Width      := Width;
Result.Height     := Height;
Result.Max        := Max;
Result.Color      := $FFFFFF;
if Texture = nil Then
  Result.Font.Color := $000000;
End;

function CreateProgressBar;
Begin
INC( LastZ );
Result          := CGUIProgressBar.Create;
if Parent = nil Then
  Result.Parent := GlobalParent
else
  Result.Parent := Parent;
Result.Z        := LastZ;

Result.Texture    := Texture;
Result.Font.fType := Font;
Result.X          := X;
Result.Y          := Y;
Result.Width      := Width;
Result.Height     := Height;
Result.Max        := Max;
Result.Color      := Color;
Result.Style      := 1;
if Texture = nil Then
  Result.Font.Color := $000000;
End;

function PointInRect( pX, pY, X, Y, Width, Height : Single ) : Boolean;
Begin
if ( pX > X ) and ( pX < X + Width  ) and
   ( pY > Y ) and ( pY < Y + Height ) Then
  Result := TRUE
else
  Result := FALSE;
End;

procedure TGUIMouse.Process;
Begin
X       := MouseEx_X;
Y       := MouseEx_Y;
Pressed := MouseEx_Down( M_LBUTTON );
End;

//----------------------------------------------------------------------------//
//-------------------------  G U I  E N G I N E  -----------------------------//
//----------------------------------------------------------------------------//

constructor CGUIEngine.Create;
Begin
inherited;
End;

procedure CGUIEngine.Draw;
  var
    b : Boolean;
Begin
CamNow := P_GetParams( gCURRENT_CAM2D );
b := CamNow^.Active;
CamNow^.Active := FALSE;

inherited;

if Mouse.Texture <> nil Then
  SSprite_Draw( Mouse.Texture,
                Mouse.X, Mouse.Y,
                Mouse.Texture.Width, Mouse.Texture.Height );

CamNow^.Active := b;
End;

procedure CGUIEngine.Process;
  var
    i : Integer;
Begin
Mouse.Process;

i := Count - 1;
while i >= 0 do
  begin
    if List[ i ].Kill Then
      begin
        Del( List[ i ].ID );
        DEC( i );
      end else
        List[ i ].Process;
    DEC( i );
  end;
End;

procedure CGUIEngine.ResetFocus;
  var
    i : Integer;
Begin
i := Count - 1;
while i >= 0 do
  begin
    if ( List[ i ] as CGUIElement ).Parent.ID = Parent.ID Then
      ( List[ i ] as CGUIElement ).Focused := FALSE;
    DEC( i );
  end;
End;

function CGUIEngine.AddElement;
  var
    i : Integer;
Begin
i            := AddNew;
List[ i ]    := Element;
List[ i ].ID := i;
( List[ i ] as CGUIElement ).Engine := Self;
Result       := i;
End;

procedure CGUIEngine.DelElement;
Begin
Del( Element.ID );
End;

//----------------------------------------------------------------------------//
//------------------------  G U I  E L E M E N T  ----------------------------//
//----------------------------------------------------------------------------//

constructor CGUIElement.Create;
Begin
inherited;
Caption := '';

Enabled := TRUE;
Visible := TRUE;

// FONT
Font.fType := nil;
Font.Scale := 1.0;
Font.Step  := 0.0;
Font.Alpha := 255;
Font.Color := $FFFFFF;

// COLOR
Color           := $888888;
AColor.Border   := $000000;
AColor.Disabled := $555555;
AColor.Focused  := $EEEEEE;
AColor.MouseIn  := $BBBBBB;
AColor.Pressed  := $FFFFFF;

// EVENTS
POnKeyDown   := nil;
POnKeyUp     := nil;
POnMouseMove := nil;
POnMouseDown := nil;
POnMouseUp   := nil;
POnProcess   := nil;
End;

procedure CGUIElement.Draw;
Begin
OnPaint;
End;

procedure CGUIElement.Process;
Begin
if not Enabled Then exit;
if not Visible Then exit;
if @POnProcess <> nil Then
  POnProcess( Self, Parent );

if PointInRect( Mouse.X, Mouse.Y, Parent.X + X, Parent.Y + Y, Width, Height ) Then
  MouseIn := TRUE
else
  MouseIn := FALSE;

if MouseIn Then
  begin
    if Mouse_Move Then
      OnMouseMove;
    OnMouseClick;
    OnMouseDown;
    OnMouseUp;

    if Mouse.Pressed Then
      begin
        Engine.ResetFocus( Parent );
        Focused := TRUE;
      end;
  end;

if Focused Then
  begin
    OnKeyDown;
    OnKeyUp;
  end;
End;

procedure CGUIElement.Show;
Begin
Visible := TRUE;
End;

procedure CGUIElement.OnPaint;
Begin
if @POnPaint <> nil Then
  POnPaint( Self, Parent );
End;

procedure CGUIElement.OnShow;
Begin
if @POnShow <> nil Then
  POnShow( Self, Parent );
End;

procedure CGUIElement.OnKeyDown;
Begin
if @POnKeyDown <> nil Then
  begin
    if Key_LastDown <> KeyLastDown Then
      begin
        KeyLastDown := Key_LastDown;
        POnKeyDown( KeyLastDown, Self, Parent );
      end else
        if Key_Down( KeyLastDown ) Then
          begin
            KeyLastDown := Key_LastDown;
            POnKeyDown( KeyLastDown, Self, Parent );
          end;
  end;
End;

procedure CGUIElement.OnKeyUp;
Begin
if @POnKeyUp <> nil Then
  begin
    if Key_LastPress <> KeyLastUp Then
      begin
        KeyLastUp := Key_LastPress;
        POnKeyUp( KeyLastUp, Self, Parent );
      end else
        if Key_Press( KeyLastUp ) Then
          begin
            KeyLastUp := Key_LastPress;
            POnKeyUp( KeyLastUp, Self, Parent );
          end;
  end;
End;

procedure CGUIElement.OnMouseMove;
Begin
if @POnMouseMove <> nil Then
  POnMouseMove( Mouse.X - Parent.X, Mouse.Y - Parent.Y, Self, Parent );
End;

procedure CGUIElement.OnMouseClick;
Begin
if @POnMouseClick <> nil Then
  begin
    if Mouse_Click( M_LBUTTON ) Then
      POnMouseClick( M_LBUTTON, Self, Parent );
    if Mouse_Click( M_RBUTTON ) Then
      POnMouseClick( M_RBUTTON, Self, Parent );
    if Mouse_Click( M_MBUTTON ) Then
      POnMouseClick( M_MBUTTON, Self, Parent );
  end;
End;

procedure CGUIElement.OnMouseDown;
Begin
if @POnMouseDown <> nil Then
  begin
    if Mouse_Down( M_LBUTTON ) Then
      POnMouseDown( M_LBUTTON, Self, Parent );
    if Mouse_Down( M_RBUTTON ) Then
      POnMouseDown( M_RBUTTON, Self, Parent );
    if Mouse_Down( M_MBUTTON ) Then
      POnMouseDown( M_MBUTTON, Self, Parent );
  end;
End;

procedure CGUIElement.OnMouseUp;
Begin
if @POnMouseUp <> nil Then
  begin
    if Mouse_Press( M_LBUTTON ) Then
      POnMouseUp( M_LBUTTON, Self, Parent );
    if Mouse_Press( M_RBUTTON ) Then
      POnMouseUp( M_RBUTTON, Self, Parent );
    if Mouse_Press( M_MBUTTON ) Then
      POnMouseUp( M_MBUTTON, Self, Parent );
  end;
End;

//----------------------------------------------------------------------------//
//--------------------------  G U I  I M A G E  ------------------------------//
//----------------------------------------------------------------------------//

procedure CGUIImage.Draw;
Begin
if not Visible Then exit;
if Texture = nil Then exit;
inherited;

if Enabled Then
  SSprite_Draw( Texture, X + Parent.X, Y + Parent.Y, Width, Height, Alpha, Angle, FX )
else
  begin
    FX := FX_DEFAULT or FX_COLORMIX;
    FX_SetColorMix( AColor.Disabled );
    SSprite_Draw( Texture, X, Y, Width, Height, Alpha, Angle, FX );
    FX := FX_DEFAULT;
  end;
End;

procedure CGUIImage.Process;
Begin
if not Enabled Then exit;
if not Visible Then exit;
inherited;

if AutoSize Then
  begin
    Width  := Texture.Width;
    Height := Texture.Height;
  end;
End;

//----------------------------------------------------------------------------//
//--------------------------  G U I  L A B E L  ------------------------------//
//----------------------------------------------------------------------------//

procedure CGUILabel.Draw;
Begin
if not Visible Then exit;
inherited;

Text_Draw( Font.fType,
           M_Round( Parent.X + X ), M_Round( Parent.Y + Y ),
           PChar( Caption ), Font.Scale, Font.Step, Font.Alpha, Font.Color );
End;

procedure CGUILabel.Process;
Begin
if not Enabled Then exit;
if not Visible Then exit;
inherited;

Width  := Text_GetWidth( Font.fType, PChar( Caption ), Font.Scale, Font.Step );
Height := Font.fType.Height * Font.Scale;
End;

//----------------------------------------------------------------------------//
//-------------------------  G U I  B U T T O N  -----------------------------//
//----------------------------------------------------------------------------//

procedure CGUIButton.Draw;
Begin
if not Visible Then exit;
inherited;
X := X + Parent.X;
Y := Y + Parent.Y;

if Texture <> nil Then
  begin
    if Enabled Then
      begin
        if MouseIn and Mouse.Pressed Then
          begin
            ASprite_Draw( Texture, X, Y, Width, Height, 3, Alpha, FX );
          end else
            if MouseIn Then
              begin
                ASprite_Draw( Texture, X, Y, Width, Height, 2, Alpha, FX );
              end else
                ASprite_Draw( Texture, X, Y, Width, Height, 1, Alpha, FX );
      end else
        ASprite_Draw( Texture, X, Y, Width, Height, 4, Alpha, FX );
  end else
    begin
      if Enabled Then
        begin
          if MouseIn and Mouse.Pressed Then
            begin
              Pr2D_Rect( X, Y, Width, Height, AColor.Pressed, Alpha, PR_FILL )
            end else
              if MouseIn Then
                begin
                  Pr2D_Rect( X, Y, Width, Height, AColor.MouseIn, Alpha, PR_FILL )
                end else
                  Pr2D_Rect( X, Y, Width, Height, Color, Alpha, PR_FILL );
        end else
          Pr2D_Rect( X, Y, Width, Height, AColor.Disabled, Alpha, PR_FILL );
      if Focused Then
        Pr2D_Rect( X, Y, Width, Height, AColor.Focused, Alpha, PR_DEFAULT )
      else
        Pr2D_Rect( X, Y, Width, Height, AColor.Border, Alpha, PR_DEFAULT );
    end;

if Caption <> '' Then
  begin
    Text_Draw( Font.fType,
               M_Round( X + Width / 2 - Text_GetWidth( Font.fType, PChar( Caption ), Font.Scale, Font.Step ) / 2 ),
               M_Round( Y + Height / 2 - Font.fType.Height / 2 ),
               PChar( Caption ), Font.Scale, Font.Step, Font.Alpha, Font.Color );
  end;

X := X - Parent.X;
Y := Y - Parent.Y;
End;

procedure CGUIButton.Process;
Begin
if not Enabled Then exit;
if not Visible Then exit;
inherited;
End;

//----------------------------------------------------------------------------//
//-----------------------  G U I  C H E C K B O X  ---------------------------//
//----------------------------------------------------------------------------//

procedure CGUICheckBox.Draw;
Begin
if not Visible Then exit;
inherited;
X := X + Parent.X;
Y := Y + Parent.Y;

if Texture <> nil Then
  begin
    if Enabled Then
      begin
        if MouseIn Then
          begin
            if Checked Then
              ASprite_Draw( Texture, X, Y, Width, Height, 4, Alpha, FX )
            else
              ASprite_Draw( Texture, X, Y, Width, Height, 3, Alpha, FX );
          end else
            if Checked Then
              begin
                ASprite_Draw( Texture, X, Y, Width, Height, 2, Alpha, FX );
              end else
                ASprite_Draw( Texture, X, Y, Width, Height, 1, Alpha, FX );
      end else
        ASprite_Draw( Texture, X, Y, Width, Height, 5, Alpha, FX );
  end else
    begin
      if Enabled Then
        begin
          if MouseIn Then
            begin
              Pr2D_Rect( X, Y, Width, Height, AColor.MouseIn, Alpha, PR_FILL );
              Pr2D_Rect( X, Y, Width, Height, AColor.Pressed, Alpha, PR_DEFAULT );
            end else
              begin
                Pr2D_Rect( X, Y, Width, Height, Color, Alpha, PR_FILL );
              end;
        end else
          Pr2D_Rect( X, Y, Width, Height, AColor.Disabled, Alpha, PR_FILL );
      if not MouseIn Then
        if Focused Then
          Pr2D_Rect( X, Y, Width, Height, AColor.Focused, Alpha, PR_DEFAULT )
        else
          Pr2D_Rect( X, Y, Width, Height, AColor.Border, Alpha, PR_DEFAULT );
      if Checked Then
        Pr2D_Rect( X + 3, Y + 3, Width - 6, Height - 6, AColor.Border, Alpha, PR_FILL );
    end;

if Caption <> '' Then
  begin
    Text_Draw( Font.fType,
               M_Round( X + Width + Font.fType.Sizes[ 0 ].Width ),
               M_Round( Y + Height / 2 - Font.fType.Height / 2 ),
               PChar( Caption ), Font.Scale, Font.Step, Font.Alpha, Font.Color );
  end;

X := X - Parent.X;
Y := Y - Parent.Y;
End;

procedure CGUICheckBox.Process;
Begin
if not Enabled Then exit;
if not Visible Then exit;
inherited;

if MouseIn Then
  if Mouse_Click( M_LBUTTON ) or Mouse_DblClick( M_LBUTTON ) Then
    begin
      Checked := not Checked;
      OnChange;
    end;
End;

procedure CGUICheckBox.OnChange;
Begin
if @POnChange <> nil Then
  POnChange( Checked, Self, Parent );
End;

//----------------------------------------------------------------------------//
//---------------------------  G U I  E D I T  -------------------------------//
//----------------------------------------------------------------------------//

procedure CGUIEdit.Draw;
  var
    t : Single;
Begin
if not Visible Then exit;
inherited;
X := X + Parent.X;
Y := Y + Parent.Y;

LineAlpha := M_Round( abs( Cos( LineTime ) ) * 255 );

if Texture <> nil Then
  begin
    SSprite_Draw( Texture, X, Y, Width, Height, Alpha );
  end else
    begin
      if Enabled Then
        begin
          Pr2D_Rect( X, Y, Width, Height, Color, Alpha, PR_FILL );
          Pr2D_Rect( X, Y, Width, Height, AColor.Border, Alpha, PR_DEFAULT );
        end else
          begin
            Pr2D_Rect( X, Y, Width, Height, AColor.Disabled, Alpha, PR_FILL );
            Pr2D_Rect( X, Y, Width, Height, AColor.Border, Alpha, PR_DEFAULT );
          end;
    end;

if Caption <> '' Then
  begin
    Text_Draw( Font.fType,
               M_Round( X + 2 ),
               M_Round( Y + Height / 2 - Font.fType.Height / 2 ),
               PChar( Caption ), Font.Scale, Font.Step, Font.Alpha, Font.Color );
  end;
if Focused Then
  begin
    t := Text_GetWidth( Font.fType, PChar( Caption ), Font.Scale, Font.Step );
    Pr2D_Line( X + 2 + t, Y + 2,
               X + 2 + t, Y - 2 + Height,
               AColor.Border, LineAlpha );
  end;

X := X - Parent.X;
Y := Y - Parent.Y;
End;

procedure CGUIEdit.Process;
Begin
if not Enabled Then exit;
if not Visible Then exit;
inherited;

if Mouse.Pressed and MouseIn Then
  begin
    Key_MaxReadText( Max );
    Key_BeginReadText( PChar( Caption ) );
  end;

if Focused Then
  Caption := Key_EndReadText;

LineTime := LineTime + 0.1;
End;

//----------------------------------------------------------------------------//
//--------------------  G U I  P R O G R E S S B A R  ------------------------//
//----------------------------------------------------------------------------//

procedure CGUIProgressBar.OnChange;
Begin
if @POnChange <> nil Then
  POnChange( Progress, Self, Parent );
End;

procedure CGUIProgressBar.Draw;
  var
    Rect : oeRect;
Begin
if not Visible Then exit;
inherited;
X := X + Parent.X;
Y := Y + Parent.Y;

if Progress > Max Then Progress := Max;
if Texture <> nil Then
  begin
    if Style = 0 Then
      begin
        ASprite_Draw( Texture, X, Y, Width, Height, 1, Alpha );
        ASprite_Draw( Texture, X, Y, Progress * ( Width / Max ), Height, 2, Alpha );
      end else
        begin
          Rect.X      := 0;
          Rect.Y      := Texture.Height / 2;
          Rect.Height := Rect.Y;
          Rect.Width  := Progress * ( Texture.Width / Max );
          if ( Progress > 0 ) and ( Width * ( Progress / Max ) >= 1 ) then
               CSprite_Draw( Texture, X, Y, Progress * ( Width / Max ), Height, @Rect, Alpha );
        end;
  end else
    begin
      if Enabled Then
        begin
          Pr2D_Rect( X, Y, ( Width / Max ) * Progress, Height, Color, Alpha, PR_FILL );
          Pr2D_Rect( X, Y, Width, Height, AColor.Border, Alpha, PR_DEFAULT );
        end else
          begin
            Pr2D_Rect( X, Y, ( Width / Max ) * Progress, Height, AColor.Disabled, Alpha, PR_FILL );
            Pr2D_Rect( X, Y, Width, Height, AColor.Border, Alpha, PR_DEFAULT );
          end;
    end;
    
X := X - Parent.X;
Y := Y - Parent.Y;
End;

procedure CGUIProgressBar.Process;
Begin
if not Enabled Then exit;
if not Visible Then exit;
inherited;

OnChange;
End;

initialization
  GUI          := CGUIEngine.Create;
  GlobalParent := CGUIElement.Create;
  GlobalParent.Enabled := FALSE;
  GlobalParent.Visible := FALSE;
  GlobalParent.X := 0;
  GlobalParent.Y := 0;
end.